/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#include <QWidget>
#include <QDebug>
#include <QVBoxLayout>
#include <QApplication>
#include "defaultapp.h"
#include "common.h"

#define BROWSERTYPE "x-scheme-handler/http"
#define MAILTYPE    "x-scheme-handler/mailto"
#define IMAGETYPE   "image/png"
#define AUDIOTYPE   "audio/x-vorbis+ogg"
#define VIDEOTYPE   "video/mp4"
#define TEXTTYPE    "text/plain"

#define DESKTOPPATH "/usr/share/applications/"

#define LOCAL_CONFIG_DIR "/.config/"
#define LOCAL_APP_DIR "/.local/share/applications/"
#define SYSTEM_CONFIG_DIR "/usr/share/applications/"

DefaultApp::DefaultApp() : mFirstLoad(true)
{
    pluginName = tr("Default App");
    pluginType = APPLICATION;
}

DefaultApp::~DefaultApp() {
    if (!mFirstLoad) {

    }
}

QString DefaultApp::plugini18nName() {
    return pluginName;
}

int DefaultApp::pluginTypes() {
    return pluginType;
}

QWidget *DefaultApp::pluginUi() {
    if (mFirstLoad) {
        mFirstLoad = false;
        pluginWidget = new QWidget;
        pluginWidget->setAttribute(Qt::WA_DeleteOnClose);

        mDefaultString = tr("No program available");
        mSelectString = tr("Choose default app");
        initUi(pluginWidget);
        initSearchText();
        initDefaultUI();
        initSlots();
        connectToServer();
//        watchFileChange();
    }
    return pluginWidget;
}

const QString DefaultApp::name() const {

    return QStringLiteral("Defaultapp");
}

bool DefaultApp::isShowOnHomePage() const
{
    return true;
}

QIcon DefaultApp::icon() const
{
    return QIcon::fromTheme("ukui-defaultapp-symbolic");
}

bool DefaultApp::isEnable() const
{
    return true;
}

/* 初始化整体UI布局 */
void DefaultApp::initUi(QWidget *widget)
{
    QVBoxLayout *mverticalLayout = new QVBoxLayout(widget);
    mverticalLayout->setSpacing(8);
    mverticalLayout->setContentsMargins(0, 0, 0, 0);

    TitleLabel *mTitleLabel = new TitleLabel(widget);
    mTitleLabel->setText(QApplication::translate("DefaultAppWindow", "Select Default Application", nullptr));
    SettingGroup *mDefaultFrame = new SettingGroup(widget);

    mBrowserFrame = new ComboxWidget(tr("Browser"));
    mMailFrame = new ComboxWidget(tr("Mail"));
    mImageFrame = new ComboxWidget(tr("Image Viewer"));
    mAudioFrame = new ComboxWidget(tr("Audio Player"));
    mVideoFrame = new ComboxWidget(tr("Video Player"));
    mTextFrame = new ComboxWidget(tr("Text Editor"));

    // 恢复默认
    mResetFrame = new PushButtonWidget(tr("Reset default apps to system recommended apps"));
    mResetFrame->setButtonText(tr("Reset"));

    mDefaultFrame->addWidget(mBrowserFrame);
    mDefaultFrame->addWidget(mMailFrame);
    mDefaultFrame->addWidget(mImageFrame);
    mDefaultFrame->addWidget(mAudioFrame);
    mDefaultFrame->addWidget(mVideoFrame);
    mDefaultFrame->addWidget(mTextFrame);
    mDefaultFrame->addWidget(mResetFrame);

    mverticalLayout->addWidget(mTitleLabel);
    mverticalLayout->addWidget(mDefaultFrame);
    mverticalLayout->addStretch();
}

/* 建立对应的信号槽 */
void DefaultApp::initSlots() {
    connect(mBrowserFrame, &ComboxWidget::currentIndexChanged, this, &DefaultApp::browserComBoBox_changed_cb);
    connect(mMailFrame, &ComboxWidget::currentIndexChanged, this, &DefaultApp::mailComBoBox_changed_cb);
    connect(mImageFrame, &ComboxWidget::currentIndexChanged, this, &DefaultApp::imageComBoBox_changed_cb);
    connect(mAudioFrame, &ComboxWidget::currentIndexChanged, this, &DefaultApp::audioComBoBox_changed_cb);
    connect(mVideoFrame, &ComboxWidget::currentIndexChanged, this, &DefaultApp::videoComBoBox_changed_cb);
    connect(mTextFrame, &ComboxWidget::currentIndexChanged, this, &DefaultApp::textComBoBox_changed_cb);
    connect(mResetFrame, &PushButtonWidget::clicked, this, &DefaultApp::reset);
}

/* 初始化各默认应用 */
void DefaultApp::initDefaultUI() {

    // 若默认应用被卸载，显示"选择默认应用"，若不存在其它应用，则显示"无可用程序"
    QTime timedebuge;//声明一个时钟对象
    timedebuge.start();//开始计时
    mBrowserFrame->blockSignals(true);
    mImageFrame->blockSignals(true);
    mMailFrame->blockSignals(true);
    mAudioFrame->blockSignals(true);
    mVideoFrame->blockSignals(true);
    mTextFrame->blockSignals(true);

    // BROWSER
    initDefaultAppInfo(BROWSERTYPE,mBrowserFrame);

    // IMAGE
    initDefaultAppInfo(IMAGETYPE,mImageFrame);

    // MAIL
    initDefaultAppInfo(MAILTYPE,mMailFrame);

    // AUDIO
    initDefaultAppInfo(AUDIOTYPE,mAudioFrame);

    // VIDEO
    initDefaultAppInfo(VIDEOTYPE,mVideoFrame);

    // TEXT
    initDefaultAppInfo(TEXTTYPE,mTextFrame);

    mBrowserFrame->blockSignals(false);
    mImageFrame->blockSignals(false);
    mMailFrame->blockSignals(false);
    mAudioFrame->blockSignals(false);
    mVideoFrame->blockSignals(false);
    mTextFrame->blockSignals(false);
}

/* 添加搜索索引 */
void DefaultApp::initSearchText() {
    //~ contents_path /Defaultapp/Browser
    //~ contents_path /Defaultapp/Mail
    //~ contents_path /Defaultapp/Image Viewer
    //~ contents_path /Defaultapp/Audio Player
    //~ contents_path /Defaultapp/Video Player
    //~ contents_path /Defaultapp/Text Editor
}

/* 设置各默认应用初始状态 */
void DefaultApp::initDefaultAppInfo(const char* type, ComboxWidget *widget)
{
    widget->comboBox()->clear();
    QString currentapp(getDefaultAppId(type));
    if (!currentapp.isEmpty()) {
        QByteArray ba = QString(DESKTOPPATH + currentapp).toUtf8();
        GDesktopAppInfo * info = g_desktop_app_info_new_from_filename(ba.constData());
        QString appname = g_app_info_get_name(G_APP_INFO(info));
        const char * iconname = g_icon_to_string(g_app_info_get_icon(G_APP_INFO(info)));
        QIcon appicon;
        appicon = QIcon::fromTheme(QString(QLatin1String(iconname)),
                                   QIcon(QString("/usr/share/pixmaps/"+QString(QLatin1String(iconname))
                                                 +".png")));
        widget->comboBox()->addItem(appicon, appname, currentapp);
    } else {
        widget->comboBox()->addItem(mSelectString);
        widget->setCurrentText(mSelectString);
        mAppCount++;
    }

    // 将可用程序加入列表
    QtConcurrent::run([=]() {
        QTime timedebuge;//声明一个时钟对象
        timedebuge.start();//开始计时
        QString current(getDefaultAppId(type));
        QVector<AppList> list = getAppIdList(type);
        if (!list.isEmpty()) {
            for (int i = 0; i < list.size(); i++) {
                QString single(list[i].strAppid);
                if (type == VIDEOTYPE && single == "kylin-music.desktop") //屏蔽视频播放器里面的音乐
                    continue;
                QByteArray ba = QString(DESKTOPPATH + single).toUtf8();
                GDesktopAppInfo * info = g_desktop_app_info_new_from_filename(ba.constData());
                QString appname = g_app_info_get_name(G_APP_INFO(info));
                const char * iconname = g_icon_to_string(g_app_info_get_icon(G_APP_INFO(info)));
                QIcon appicon;
                appicon = QIcon::fromTheme(QString(QLatin1String(iconname)),
                                           QIcon(QString("/usr/share/pixmaps/"+QString(QLatin1String(iconname))
                                                         +".png")));
                if (current == single) {
                    continue;
                }
                if (appname.isEmpty())
                    continue;
                widget->comboBox()->addItem(appicon, appname, single);
            }
        }
        if (widget->comboBox()->count() == 1 && currentapp.isEmpty()) {
            mAppCount--;
            widget->comboBox()->clear();
            widget->comboBox()->addItem(mDefaultString);
            widget->setCurrentText(mDefaultString);
        }
    });
}

/* 重构界面 */
void DefaultApp::resetUi()
{
    mBrowserFrame->blockSignals(true);
    mAudioFrame->blockSignals(true);
    mImageFrame->blockSignals(true);
    mTextFrame->blockSignals(true);
    mMailFrame->blockSignals(true);
    mVideoFrame->blockSignals(true);
    initDefaultUI();
    mBrowserFrame->blockSignals(false);
    mAudioFrame->blockSignals(false);
    mImageFrame->blockSignals(false);
    mTextFrame->blockSignals(false);
    mMailFrame->blockSignals(false);
    mVideoFrame->blockSignals(false);
}

/* BROWSER SLOT */
void DefaultApp::browserComBoBox_changed_cb(int index) {
   QtConcurrent::run([=] {
        QTime timedebuge;//声明一个时钟对象
        timedebuge.start();//开始计时
        QString appid = mBrowserFrame->comboBox()->itemData(index).toString();
        QByteArray ba = appid.toUtf8(); // QString to char *
        setWebBrowsersDefaultProgram(ba.data());
        findSelectItem(mBrowserFrame->comboBox());
        Common::buriedSettings(name(), "the default browser", QString("settings"), ba.data());
        qDebug()<<"browserComBoBox_changed_cb线程耗时："<<timedebuge.elapsed()<<"ms";//输出计时
    });
}

/* MAIL SLOT */
void DefaultApp::mailComBoBox_changed_cb(int index) {
    QtConcurrent::run([=] {
        QTime timedebuge;//声明一个时钟对象
        timedebuge.start();//开始计时
        QString appid = mMailFrame->comboBox()->itemData(index).toString();
        QByteArray ba = appid.toUtf8(); // QString to char *
        setMailReadersDefaultProgram(ba.data());
        findSelectItem(mMailFrame->comboBox());
        Common::buriedSettings(name(), "the defaultapp to open mail", QString("settings"), ba.data());
        qDebug()<<"mailComBoBox_changed_cb线程耗时："<<timedebuge.elapsed()<<"ms";//输出计时
    });
}

/* IMAGE SLOT */
void DefaultApp::imageComBoBox_changed_cb(int index) {

    QtConcurrent::run([=] {
        QTime timedebuge;//声明一个时钟对象
        timedebuge.start();//开始计时
        QString appid = mImageFrame->comboBox()->itemData(index).toString();
        QByteArray ba = appid.toUtf8(); // QString to char *
        setImageViewersDefaultProgram(ba.data());
        findSelectItem(mImageFrame->comboBox());
        Common::buriedSettings(name(), "the defaultapp to open image", QString("settings"), ba.data());
        qDebug()<<"imageComBoBox_changed_cb线程耗时："<<timedebuge.elapsed()<<"ms";//输出计时
    });
}

/* AUDIO SLOT */
void DefaultApp::audioComBoBox_changed_cb(int  index) {

    QtConcurrent::run([=] {
        QTime timedebuge;//声明一个时钟对象
        timedebuge.start();//开始计时
        QString appid = mAudioFrame->comboBox()->itemData(index).toString();
        QByteArray ba = appid.toUtf8(); // QString to char *
        setAudioPlayersDefaultProgram(ba.data());
        findSelectItem(mAudioFrame->comboBox());
        Common::buriedSettings(name(), "the defaultapp to play audio", QString("settings"), ba.data());
        qDebug()<<"audioComBoBox_changed_cb线程耗时："<<timedebuge.elapsed()<<"ms";//输出计时
    });
}

/* VIDEO SLOT */
void DefaultApp::videoComBoBox_changed_cb(int index) {

    QtConcurrent::run([=] {
        QTime timedebuge;//声明一个时钟对象
        timedebuge.start();//开始计时
        QString appid = mVideoFrame->comboBox()->itemData(index).toString();
        QByteArray ba = appid.toUtf8(); // QString to char *
        setVideoPlayersDefaultProgram(ba.data());
        findSelectItem(mVideoFrame->comboBox());
        Common::buriedSettings(name(), "the defaultapp to play video", QString("settings"), ba.data());
        qDebug()<<"videoComBoBox_changed_cb线程耗时："<<timedebuge.elapsed()<<"ms";//输出计时
    });
}

/* TEXT SLOT */
void DefaultApp::textComBoBox_changed_cb(int index) {

    QtConcurrent::run([=] {
        QTime timedebuge;//声明一个时钟对象
        timedebuge.start();//开始计时
        QString appid = mTextFrame->comboBox()->itemData(index).toString();
        QByteArray ba = appid.toUtf8(); // QString to char *
        setTextEditorsDefautlProgram(ba.data());
        findSelectItem(mTextFrame->comboBox());
        Common::buriedSettings(name(), "the defaultapp to open text", QString("settings"), ba.data());
        qDebug()<<"textComBoBox_changed_cb线程耗时："<<timedebuge.elapsed()<<"ms";//输出计时
    });
}

/* 获取默认应用的应用名 */
QString DefaultApp::getDefaultAppId(const char *contentType) {
    QString localfile = QDir::homePath() + LOCAL_CONFIG_DIR + "mimeapps.list";
    QString systemfile = SYSTEM_CONFIG_DIR + QString("ukui-mimeapps.list");
    if (QFile(localfile).exists()) {
        QSettings* mimeappFile = new QSettings(localfile, QSettings::IniFormat);
        mimeappFile->setIniCodec("utf-8");
        QString str = mimeappFile->value(QString("Default Applications/%1").arg(contentType)).toString();
        if (!str.isEmpty()) {
            if (QFile(SYSTEM_CONFIG_DIR +str).exists()) {
                return str;
            } else if (QFile(QDir::homePath() + LOCAL_APP_DIR + str).exists()) {
                QDBusInterface ukccDbus("com.control.center.qt.systemdbus",
                                        "/",
                                        "com.control.center.interface",
                                        QDBusConnection::systemBus());

                QDBusReply<bool> reply = ukccDbus.call("copyFile", QDir::homePath() + LOCAL_APP_DIR + str, SYSTEM_CONFIG_DIR +str);
                if (reply)
                    return str;
            }
        }
        delete mimeappFile;
        mimeappFile = nullptr;
    }
    if (QFile(systemfile).exists()) {
        QSettings* mimeappFile = new QSettings(systemfile, QSettings::IniFormat);
        mimeappFile->setIniCodec("utf-8");
        QString str = mimeappFile->value(QString("Default Applications/%1").arg(contentType)).toString();
        if (!str.isEmpty()) {
            if (QFile(SYSTEM_CONFIG_DIR +str).exists()) {
                return str;
            } else if (QFile(QDir::homePath() + LOCAL_APP_DIR + str).exists()) {
                QDBusInterface ukccDbus("com.control.center.qt.systemdbus",
                                        "/",
                                        "com.control.center.interface",
                                        QDBusConnection::systemBus());

                QDBusReply<bool> reply = ukccDbus.call("copyFile", QDir::homePath() + LOCAL_APP_DIR + str, SYSTEM_CONFIG_DIR +str);
                if (reply)
                    return str;
            }
        }
        delete mimeappFile;
        mimeappFile = nullptr;
    }
    return QString("");
}

/* 获取可用应用列表 */
QVector<AppList> DefaultApp::getAppIdList(const char *contentType) {
    QVector<AppList> appList;
    appList.clear();
    QVector<Appinfo> appinfo = _getAppList(contentType);
    if (!appinfo.isEmpty()) {
        for(int j = 0; j < appinfo.size(); j++) {
            const char *id = g_app_info_get_id(appinfo[j].item);
            if (id != NULL) {
                AppList al;
                al.strAppid = QString(id);
                appList.append(al);
            }
        }
    }
    return appList;
}

/* 获取可用应用信息列表 */
QVector<Appinfo> DefaultApp::_getAppList(const char *contentType) {
    GList *applist;
    applist = g_app_info_get_all_for_type(contentType);
    GAppInfo * item;
    QVector<Appinfo> appinfo;
    appinfo.clear();

    if (applist != NULL) {
        int len = g_list_length(applist);

        //获取应用列表
        for (int index=0; index < len; index++) {
            item = (GAppInfo*) g_list_nth_data(applist, index);
            Appinfo ai;
            ai.item = item;
            appinfo.append(ai);
        }
    }
    return appinfo;
}

/* Set Default Browser */
bool DefaultApp::setWebBrowsersDefaultProgram(char * appid) {
    qDebug() << __func__ << __LINE__;

    const char * content_type = "x-scheme-handler/http";
    QVector<Appinfo> appinfo = _getAppList(content_type);
    bool judge = false;
    if (!appinfo.isEmpty()) {
        for(int i = 0; i < appinfo.size(); i++) {
            const char *id = g_app_info_get_id(appinfo[i].item);
            int result = strcmp(id,appid);
            if (0 == result) {
                GAppInfo *appitem=appinfo[i].item;
                gboolean ret1=g_app_info_set_as_default_for_type(appitem, "x-scheme-handler/http", NULL);
                gboolean ret2=g_app_info_set_as_default_for_type(appitem, "x-scheme-handler/https", NULL);
                gboolean ret3=g_app_info_set_as_default_for_type(appitem, "x-scheme-handler/about", NULL);
                gboolean ret4=g_app_info_set_as_default_for_type(appitem, "text/html", NULL);
                if(ret1 && ret2 && ret3 && ret4)
                    judge=true;
                break;
            }
        }
    }
    qDebug() << __func__ << __LINE__;
    return judge;
}

/* Set Default Mail */
bool DefaultApp::setMailReadersDefaultProgram(char *appid) {
    const char *content_type="x-scheme-handler/mailto";
    QVector<Appinfo> appinfo = _getAppList(content_type);

    bool judge = false;
    if (!appinfo.isEmpty()) {
        for (int i = 0; i < appinfo.size(); i++) {
            const char * id = g_app_info_get_id(appinfo[i].item);
            int result=strcmp(id,appid);
            if (0 == result) {
                GAppInfo *appitem=appinfo[i].item;
                gboolean ret1=g_app_info_set_as_default_for_type(appitem, "x-scheme-handler/mailto", NULL);
                gboolean ret2=g_app_info_set_as_default_for_type(appitem, "application/x-extension-eml", NULL);
                gboolean ret3=g_app_info_set_as_default_for_type(appitem, "message/rfc822", NULL);
                if(ret1 && ret2 && ret3)
                    judge=true;
                break;
            }
        }
    }
    return judge;
}

/* Set Default Image */
bool DefaultApp::setImageViewersDefaultProgram(char *appid) {
    const char *content_type="image/png";
    QVector<Appinfo> appinfo = _getAppList(content_type);

    bool judge = false;
    if (!appinfo.isEmpty()) {
        for (int i=0; i < appinfo.size();i++){
            const char *id = g_app_info_get_id(appinfo[i].item);
            int result = strcmp(id, appid);
            if (0 == result) {
                GAppInfo *appitem=appinfo[i].item;
                gboolean ret1 = g_app_info_set_as_default_for_type(appitem, "image/bmp", NULL);
                gboolean ret2 = g_app_info_set_as_default_for_type(appitem, "image/gif", NULL);
                gboolean ret3 = g_app_info_set_as_default_for_type(appitem, "image/jpeg", NULL);
                gboolean ret4 = g_app_info_set_as_default_for_type(appitem, "image/png", NULL);
                gboolean ret5 = g_app_info_set_as_default_for_type(appitem, "image/tiff", NULL);
                gboolean ret6 = g_app_info_set_as_default_for_type(appitem, "image/webp", NULL);
                gboolean ret7 = g_app_info_set_as_default_for_type(appitem, "image/x-xbitmap", NULL);
                gboolean ret8 = g_app_info_set_as_default_for_type(appitem, "image/x-xpixmap", NULL);
                gboolean ret9 = g_app_info_set_as_default_for_type(appitem, "image/x-icon", NULL);
                if(ret1 && ret2 && ret3 && ret4 && ret5 && ret6 && ret7 && ret8 && ret9)
                    judge=true;
                break;
            }
        }
    }

    return judge;
}

/* Set Default Video */
bool DefaultApp::setVideoPlayersDefaultProgram(char *appid) {
    const char *content_type = "video/x-ogm+ogg";
    QVector<Appinfo> appinfo = _getAppList(content_type);

    bool judge = false;
    if (!appinfo.isEmpty()) {
        for(int i = 0; i < appinfo.size(); i++) {
            const char *id = g_app_info_get_id(appinfo[i].item);
            int result = strcmp(id,appid);
            if (0 == result) {
                GAppInfo *appitem = appinfo[i].item;
                gboolean ret1 = g_app_info_set_as_default_for_type(appitem, "video/mp4", NULL);
                gboolean ret2 = g_app_info_set_as_default_for_type(appitem, "video/mpeg", NULL);
                gboolean ret3 = g_app_info_set_as_default_for_type(appitem, "video/mp2t", NULL);
                gboolean ret4 = g_app_info_set_as_default_for_type(appitem, "video/msvideo", NULL);
                gboolean ret5 = g_app_info_set_as_default_for_type(appitem, "video/quicktime", NULL);
                gboolean ret6 = g_app_info_set_as_default_for_type(appitem, "video/webm", NULL);
                gboolean ret7 = g_app_info_set_as_default_for_type(appitem, "video/x-avi", NULL);
                gboolean ret8 = g_app_info_set_as_default_for_type(appitem, "video/x-flv", NULL);
                gboolean ret9 = g_app_info_set_as_default_for_type(appitem, "video/x-matroska", NULL);
                gboolean ret10 = g_app_info_set_as_default_for_type(appitem, "video/x-mpeg", NULL);
                gboolean ret11 = g_app_info_set_as_default_for_type(appitem, "video/x-ogm+ogg", NULL);
                gboolean ret12 = g_app_info_set_as_default_for_type(appitem, "video/rm", NULL);
                gboolean ret13 = g_app_info_set_as_default_for_type(appitem, "video/3pg", NULL);
                gboolean ret14 = g_app_info_set_as_default_for_type(appitem, "video/asf", NULL);
                gboolean ret15 = g_app_info_set_as_default_for_type(appitem, "video/3gp", NULL);
                gboolean ret16 = g_app_info_set_as_default_for_type(appitem, "video/3gpp", NULL);
                gboolean ret17 = g_app_info_set_as_default_for_type(appitem, "video/3gpp2", NULL);
                gboolean ret18 = g_app_info_set_as_default_for_type(appitem, "video/x-ms-afs", NULL);
                gboolean ret19 = g_app_info_set_as_default_for_type(appitem, "video/x-ms-asf", NULL);
                gboolean ret20 = g_app_info_set_as_default_for_type(appitem, "video/x-mpeg2", NULL);
                gboolean ret21 = g_app_info_set_as_default_for_type(appitem, "video/x-mpeg3", NULL);
                gboolean ret22 = g_app_info_set_as_default_for_type(appitem, "video/mp4v-es", NULL);
                gboolean ret23 = g_app_info_set_as_default_for_type(appitem, "video/x-m4v", NULL);
                gboolean ret24 = g_app_info_set_as_default_for_type(appitem, "video/divx", NULL);
                gboolean ret25 = g_app_info_set_as_default_for_type(appitem, "video/vnd.divx", NULL);
                gboolean ret26 = g_app_info_set_as_default_for_type(appitem, "video/x-msvideo", NULL);
                gboolean ret27 = g_app_info_set_as_default_for_type(appitem, "video/ogg", NULL);
                gboolean ret28 = g_app_info_set_as_default_for_type(appitem, "video/vnd.rn-realvideo", NULL);
                gboolean ret29 = g_app_info_set_as_default_for_type(appitem, "video/x-ms-wmv", NULL);
                gboolean ret30 = g_app_info_set_as_default_for_type(appitem, "video/x-ms-wmx", NULL);
                gboolean ret31 = g_app_info_set_as_default_for_type(appitem, "video/x-ms-wvxvideo", NULL);
                gboolean ret32 = g_app_info_set_as_default_for_type(appitem, "video/avi", NULL);
                gboolean ret33 = g_app_info_set_as_default_for_type(appitem, "video/x-flic", NULL);
                gboolean ret34 = g_app_info_set_as_default_for_type(appitem, "video/fli", NULL);
                gboolean ret35 = g_app_info_set_as_default_for_type(appitem, "video/x-flc", NULL);
                gboolean ret36 = g_app_info_set_as_default_for_type(appitem, "video/flv", NULL);
                gboolean ret37 = g_app_info_set_as_default_for_type(appitem, "video/x-theora", NULL);
                gboolean ret38 = g_app_info_set_as_default_for_type(appitem, "video/x-theora+ogg", NULL);
                gboolean ret39 = g_app_info_set_as_default_for_type(appitem, "video/mkv", NULL);
                gboolean ret40 = g_app_info_set_as_default_for_type(appitem, "video/x-ogm", NULL);
                gboolean ret41 = g_app_info_set_as_default_for_type(appitem, "video/vnd.mpegurl", NULL);
                gboolean ret42 = g_app_info_set_as_default_for_type(appitem, "video/dv", NULL);
                gboolean ret43 = g_app_info_set_as_default_for_type(appitem, "application/vnd.rn-realmedia", NULL);
                gboolean ret44 = g_app_info_set_as_default_for_type(appitem, "application/vnd.rn-realmedia-vbr", NULL);
                if (ret1 && ret2 && ret3 && ret4 && ret5 && ret6 && ret7 && ret8 && ret9 && ret10 && ret11 && \
                        ret12 && ret13 && ret14 && ret15 && ret16 && ret17 && ret18 && ret19 && ret20 && ret21 && \
                        ret22 && ret23 && ret24 && ret25 && ret26 && ret27 && ret28 && ret29 && ret30 && \
                        ret31 && ret32 && ret33 && ret34 && ret35 && ret36 && ret37 && ret38 && ret39 && \
                        ret40 && ret41 && ret42 && ret43 && ret44)
                    judge=true;
                break;
            }
        }
    }
    return judge;
}

/* Set Default Audio */
bool DefaultApp::setAudioPlayersDefaultProgram(char *appid) {
    const char *content_type = "audio/x-vorbis+ogg";
    QVector<Appinfo> appinfo = _getAppList(content_type);

    bool judge = false;
    if (!appinfo.isEmpty()) {
        for(int i = 0; i < appinfo.size() ;i++) {
            const char *id = g_app_info_get_id(appinfo[i].item);
            int result = strcmp(id,appid);
            if (0 == result) {
                GAppInfo *appitem=appinfo[i].item;
                gboolean ret1 = g_app_info_set_as_default_for_type(appitem, "audio/mpeg", NULL);
                gboolean ret2 = g_app_info_set_as_default_for_type(appitem, "audio/x-mpegurl", NULL);
                gboolean ret3 = g_app_info_set_as_default_for_type(appitem, "audio/x-scpls", NULL);
                gboolean ret4 = g_app_info_set_as_default_for_type(appitem, "audio/x-vorbis+ogg", NULL);
                gboolean ret5 = g_app_info_set_as_default_for_type(appitem, "audio/x-wav", NULL);
                gboolean ret6 = g_app_info_set_as_default_for_type(appitem, "audio/x-ms-wma", NULL);
                gboolean ret7 = g_app_info_set_as_default_for_type(appitem, "audio/x-flac", NULL);
                gboolean ret8 = g_app_info_set_as_default_for_type(appitem, "audio/ac3", NULL);
                gboolean ret9 = g_app_info_set_as_default_for_type(appitem, "audio/acc", NULL);
                gboolean ret10 = g_app_info_set_as_default_for_type(appitem, "audio/aac", NULL);
                gboolean ret11 = g_app_info_set_as_default_for_type(appitem, "audio/AMR", NULL);
                gboolean ret12 = g_app_info_set_as_default_for_type(appitem, "audio/x-m4r", NULL);
                gboolean ret13 = g_app_info_set_as_default_for_type(appitem, "audio/midi", NULL);
                gboolean ret14 = g_app_info_set_as_default_for_type(appitem, "audio/mp2", NULL);
                gboolean ret15 = g_app_info_set_as_default_for_type(appitem, "audio/x-wavpack", NULL);
                gboolean ret16 = g_app_info_set_as_default_for_type(appitem, "audio/x-ape", NULL);
                gboolean ret17 = g_app_info_set_as_default_for_type(appitem, "audio/x-mmf", NULL);
                gboolean ret18 = g_app_info_set_as_default_for_type(appitem, "audio/mp3", NULL);
                gboolean ret19 = g_app_info_set_as_default_for_type(appitem, "audio/flac", NULL);
                gboolean ret20 = g_app_info_set_as_default_for_type(appitem, "audio/wma", NULL);
                gboolean ret21 = g_app_info_set_as_default_for_type(appitem, "audio/amr", NULL);
                gboolean ret22 = g_app_info_set_as_default_for_type(appitem, "audio/vnd.dts", NULL);
                gboolean ret23 = g_app_info_set_as_default_for_type(appitem, "audio/basic", NULL);
                gboolean ret24 = g_app_info_set_as_default_for_type(appitem, "audio/x-aiff", NULL);
                gboolean ret25 = g_app_info_set_as_default_for_type(appitem, "audio/x-matroska", NULL);
                gboolean ret26 = g_app_info_set_as_default_for_type(appitem, "audio/x-voc", NULL);
                gboolean ret27 = g_app_info_set_as_default_for_type(appitem, "application/x-smaf", NULL);
                if(ret1 && ret2 && ret3 && ret4 && ret5 && ret6 && ret7 && ret8 && ret9 && ret10 && ret11 && ret12 && ret13 && ret14
                        && ret15 && ret16 && ret17 && ret18 && ret19 && ret20 && ret21 && ret22 && ret23 && ret24 && ret25 && ret26
                        && ret27) {
                    judge=true;
                }
                break;
            }
        }
    }
    return judge;
}

/* Set Default Text */
bool DefaultApp::setTextEditorsDefautlProgram(char *appid) {
    const char * content_type = "text/plain";
    QVector<Appinfo> appinfo = _getAppList(content_type);

    bool judge = false;
    if (!appinfo.isEmpty()) {
        for (int i = 0;i < appinfo.size(); i++) {
            const char * id = g_app_info_get_id(appinfo[i].item);
            int result = strcmp(id,appid);
            if (0 == result) {
                GAppInfo *appitem = appinfo[i].item;
                gboolean ret1 = g_app_info_set_as_default_for_type(appitem, "text/plain", NULL);
                if(ret1)
                    judge=true;
                break;
            }
        }
    }

    return judge;
}

void DefaultApp::connectToServer()
{
    QThread *NetThread = new QThread;
    MThread *NetWorker = new MThread;
    NetWorker->moveToThread(NetThread);
    connect(NetThread, &QThread::started, NetWorker, &MThread::run);
    connect(NetWorker,&MThread::keychangedsignal,this,&DefaultApp::keyChangedSlot);
    connect(NetThread, &QThread::finished, NetWorker, &MThread::deleteLater);
    NetThread->start();
}

/* 监听默认应用的改变刷新界面 */
void DefaultApp::watchFileChange()
{
    watchfile =  QDir::homePath() + "/.config/mimeapps.list";
    mConfigFileWatcher = new QFileSystemWatcher(this);
    mConfigFileWatcher->addPath(watchfile);
    connect(mConfigFileWatcher, &QFileSystemWatcher::fileChanged, this, [=](){
        if (!mComboChange)
            resetUi();
        mComboChange = false;
        mConfigFileWatcher->removePath(watchfile);
        mConfigFileWatcher->addPath(watchfile);
    });
}

void DefaultApp::findSelectItem(QComboBox *combox)
{
    if (mAppCount == 0)
        return;
    for (int i = 0; i < combox->count(); i++) {
        if (combox->itemText(i) == mSelectString) {
            combox->removeItem(i);
            mAppCount--;
            break;
        }
    }
}

void DefaultApp::keyChangedSlot(const QString &key) {
    if(key == "default-open") {
        resetUi();
    }
}

void DefaultApp::reset()
{
    Common::buriedSettings(name(), "reset defaultapp", QString("clicked"));
    QString localfile = QDir::homePath() + LOCAL_CONFIG_DIR + "mimeapps.list";
    QFile(localfile).remove();
    mAppCount = 0;
    initDefaultUI();
}
