#include "printerbtn.h"
#include <QHBoxLayout>
#include <QLabel>
#include <QVariant>
#include <QGSettings>
#include <QProcess>

#include "fixlabel.h"
#define THEME_QT_SCHEMA  "org.ukui.style"

#define ICON_QT_KEY      "icon-theme-name"

const QStringList networktype {"socket", "lpd", "dnssd", "hp:/net/", "hpfax:/net/"};

PrinterBtn::PrinterBtn(const QString info, const QString name, QWidget *parent):
    printerinfo(info),
    QPushButton(parent)
{
    this->setStyleSheet("PrinterBtn:!pressed:hover{background-color: palette(button); border-radius: 6px;}"
                        "PrinterBtn:!pressed:!hover{background-color: palette(base); border-radius: 6px;}");
    this->setProperty("useButtonPalette", true);
    this->setFlat(true);
    setMinimumSize(QSize(550, 60));
    setMaximumSize(QSize(16777215, 60));
    QHBoxLayout *mPrinterLyt = new QHBoxLayout(this);
    mPrinterLyt->setSpacing(16);

    QLabel *iconLabel = new QLabel(this);
    QIcon icon = QIcon::fromTheme(isNetPrinter(name) ? "printer-remote" : "cupsprinter");
    iconLabel->setPixmap(icon.pixmap(icon.actualSize(QSize(24, 24))));

    const QByteArray id(THEME_QT_SCHEMA);
    QGSettings *icongsettings = new QGSettings(id, QByteArray(), this);

    connect(icongsettings, &QGSettings::changed, this, [=](const QString &key) {
        if (key == "iconThemeName") {
            QIcon icon = QIcon::fromTheme(isNetPrinter(name) ? "printer-remote" : "cupsprinter");
            iconLabel->setPixmap(icon.pixmap(icon.actualSize(QSize(24, 24))));
        }
    });

    FixLabel *textLabel = new FixLabel(this);
    textLabel->setText(name);

    mPrinterLyt->addWidget(iconLabel);
    mPrinterLyt->addWidget(textLabel,Qt::AlignLeft);
}

bool PrinterBtn::isNetPrinter(QString printername)
{
    QStringList printerinfo_list = printerinfo.split("\n");
    for (QString str : printerinfo_list) {
        if (str.contains(printername)) {
           for (QString type : networktype) {
               if (str.contains(type))
                   return true;
           }
        }
    }
    return false;
}
