/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#include <QWidget>

#include "about.h"
#include "trialdialog.h"

#include <KFormat>
#include <unistd.h>
#include <QFile>
#include <QGridLayout>
#include <QPluginLoader>
#include <QEvent>
#include <QMessageBox>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pthread.h>
#include <signal.h>

#ifdef Q_OS_LINUX
#include <sys/sysinfo.h>
#elif defined(Q_OS_FREEBSD)
#include <sys/types.h>
#include <sys/sysctl.h>
#endif

#include <QProcess>
#include <QFile>
#include <QDebug>
#include <QStorageInfo>
#include <QtMath>
#include <QSvgRenderer>

const QString kAboutFile = "/usr/share/applications/kylin-user-guide.desktop";
const QString kHPFile = "/usr/share/applications/hp-document.desktop";
QStringList mIpList;

About::About() : mFirstLoad(true)
{
#ifdef MAVIS
    pluginName = tr("About and Support");
#else
    pluginName = tr("About");
#endif
    pluginType = SYSTEM;
}

About::~About()
{
    if (!mFirstLoad) {

    }
}

QString About::plugini18nName()
{
    return pluginName;
}

int About::pluginTypes()
{
    return pluginType;
}

QWidget *About::pluginUi()
{
    if (mFirstLoad) {
        mFirstLoad = false;

        pluginWidget = new QWidget;
        pluginWidget->setAttribute(Qt::WA_DeleteOnClose);
        pluginWidget->setObjectName("pluginWidget");

        initUI(pluginWidget);
        initSearchText();
        initActiveDbus();

        setConnect();
        setupVersionCompenent();
        setVersionNumCompenent();
        setupDesktopComponent();
        setHostNameCompenet();
        setupSystemVersion();
        setupKernelCompenent();
        setupSerialComponent();
        setPrivacyCompent();

        if (!mDiskFrame->isHidden())
            setupDiskCompenet();

        mNtphostName.append(QString("0.cn.pool.ntp.org"));
        mNtphostName.append(QString("1.cn.pool.ntp.org"));
        mNtphostName.append(QString("2.cn.pool.ntp.org"));
        mNtphostName.append(QString("3.cn.pool.ntp.org"));
        mNtphostName.append(QString("cn.pool.ntp.org"));
        mNtphostName.append(QString("0.tw.pool.ntp.org"));
        mNtphostName.append(QString("1.tw.pool.ntp.org"));
        mNtphostName.append(QString("2.tw.pool.ntp.org"));
        mNtphostName.append(QString("3.tw.pool.ntp.org"));
        mNtphostName.append(QString("tw.pool.ntp.org"));
        mNtphostName.append(QString("pool.ntp.org"));
        mNtphostName.append(QString("time.windows.com"));
        mNtphostName.append(QString("time.nist.gov"));
        mNtphostName.append(QString("time-nw.nist.gov"));
        mNtphostName.append(QString("asia.pool.ntp.org"));
        mNtphostName.append(QString("europe.pool.ntp.org"));
        mNtphostName.append(QString("oceania.pool.ntp.org"));
        mNtphostName.append(QString("north-america.pool.ntp.org"));
        mNtphostName.append(QString("south-america.pool.ntp.org"));
        mNtphostName.append(QString("africa.pool.ntp.org"));
        mNtphostName.append(QString("ca.pool.ntp.org"));
        mNtphostName.append(QString("uk.pool.ntp.org"));
        mNtphostName.append(QString("us.pool.ntp.org"));
        mNtphostName.append(QString("au.pool.ntp.org"));
    }
    return pluginWidget;
}

const QString About::name() const
{
    return QStringLiteral("About");
}

bool About::isShowOnHomePage() const
{
    return true;
}

QIcon About::icon() const
{
    if (QIcon::hasThemeIcon("preferences-system-details-symbolic")) {
        return QIcon::fromTheme("preferences-system-details-symbolic");
    } else {
        return QIcon();
    }
}

bool About::isEnable() const
{
    return true;
}

/* 初始化整体UI布局 */
void About::initUI(QWidget *widget)
{
    QVBoxLayout *mverticalLayout = new QVBoxLayout(widget);
    mverticalLayout->setSpacing(0);
    mverticalLayout->setContentsMargins(0, 0, 0, 0);

    QWidget *Aboutwidget = new QWidget(widget);
    Aboutwidget->setMinimumSize(QSize(550, 0));
    Aboutwidget->setMaximumSize(QSize(16777215, 16777215));

    QVBoxLayout *AboutLayout = new QVBoxLayout(Aboutwidget);
    AboutLayout->setContentsMargins(0, 0, 0, 0);
    AboutLayout->setSpacing(8);

    mTitleLabel = new TitleLabel(Aboutwidget);
    mTitleLabel->setText(tr("System Summary"));
    mPriTitleLabel = new TitleLabel(Aboutwidget);
    mPriTitleLabel->setText(tr("Privacy and agreement"));
    mHoldTitleLabel = new TitleLabel(Aboutwidget);
    mHoldTitleLabel->setText(tr("Support"));

    mInformationFrame = new QFrame(Aboutwidget);
    setFrame_Box(mInformationFrame);

    QVBoxLayout *mInformationLayout = new QVBoxLayout(mInformationFrame);
    int space = Common::isTablet() ? 32 : 16;
    mInformationLayout->setContentsMargins(space, space, space, space);

    /* LOGO */
    mLogoLabel = new FixLabel(mInformationFrame);
    mLogoLabel->setFixedSize(130, 50);

     /* 版本名称 */
    mVersionLabel_1 = new FixLabel(tr("Version"));
    mVersionLabel_2 = new LightLabel;
    mVersionFrame = createFrame(mVersionLabel_1, mVersionLabel_2);

     /* 版本号 */
    mVersionNumberLabel_1 = new FixLabel(tr("Version Number"));
    mVersionNumberLabel_2 = new LightLabel;
    mVersionNumberFrame = createFrame(mVersionNumberLabel_1, mVersionNumberLabel_2);

    /* 系统补丁版本号 */
    mPatchVersionLabel_1 = new FixLabel(tr("Patch Version"));
    mPatchVersionLabel_2 = new LightLabel;
    mPatchVersionFrame = createFrame(mPatchVersionLabel_1, mPatchVersionLabel_2);

    /* 主机名 */
    mHostNameLabel_1 = new FixLabel(tr("HostName"));
    mHostNameLabel_2 = new LightLabel;
    mHostNameLabel_3 = new LightLabel;
    mHostNameLabel_3->setFixedSize(16, 16);
    mHostNameLabel_3->setProperty("useIconHighlightEffect", 0x8);
    mHostNameLabel_3->setPixmap(QIcon::fromTheme("document-edit-symbolic").pixmap(mHostNameLabel_3->size()));
    mHostNameLabel_3->setScaledContents(true);
    mHostNameLabel_2->installEventFilter(this);
    mHostNameLabel_3->installEventFilter(this);
    mHostNameFrame = createFrame(mHostNameLabel_1, mHostNameLabel_2, mHostNameLabel_3);

    /* 内核 */
    mKernelLabel_1 = new FixLabel(tr("Kernel"));
    mKernelLabel_2 = new LightLabel;
    mKernelFrame = createFrame(mKernelLabel_1, mKernelLabel_2);

    /* CPU */
    mCpuLabel_1 = new FixLabel(tr("CPU"));
    mCpuLabel_2 = new LightLabel;
    mCpuFrame = createFrame(mCpuLabel_1, mCpuLabel_2);

    /* 内存 */
    mMemoryLabel_1 = new FixLabel(tr("Memory"));
    mMemoryLabel_2 = new LightLabel;
    mMemoryFrame = createFrame(mMemoryLabel_1, mMemoryLabel_2);

    /* 硬盘 */
    mDiskLabel_1 = new FixLabel(tr("Disk"));
    mDiskLabel_2 = new LightLabel;
    mDiskFrame = createFrame(mDiskLabel_1, mDiskLabel_2);

    /* 桌面 */
    mDesktopLabel_1 = new FixLabel(tr("Desktop"));
    mDesktopLabel_2 = new LightLabel;
    mDesktopFrame = createFrame(mDesktopLabel_1, mDesktopLabel_2);

    /* 用户名 */
    mUsernameLabel_1 = new FixLabel(tr("User"));
    mUsernameLabel_2 = new LightLabel;
    mUsernameFrame = createFrame(mUsernameLabel_1, mUsernameLabel_2);

    /* 激活信息 */
    mActivationFrame = initActiveUi();

    /* 版权所有 */
    QString currentYear("2022");
    mTipLabel = new FixLabel(QString(tr("Copyright © 2009-%1 %2. All rights reserved.")).arg(currentYear).arg(mCopyRightName), Aboutwidget);
    mTipLabel->setContentsMargins(16, 0, 0, 0);

    /* 隐私和协议 */
    mPrivacyFrame = initPrivacyUi();

    /* 授权信息和隐私协议弹窗 */
    mBtnFrame = initInfoBtnUi();

    /* Intel手册 */
    mHoldWidget = initHoldUi();

    item = new QSpacerItem(10 ,32);
    mInformationLayout->addWidget(mLogoLabel);
    mInformationLayout->addSpacing(24);
    mInformationLayout->addWidget(mVersionFrame);
    mInformationLayout->addWidget(mVersionNumberFrame);
    mInformationLayout->addWidget(mPatchVersionFrame);
    mInformationLayout->addWidget(mHostNameFrame);
    mInformationLayout->addWidget(mKernelFrame);
    mInformationLayout->addWidget(mCpuFrame);
    mInformationLayout->addWidget(mMemoryFrame);
    mInformationLayout->addWidget(mDiskFrame);
    mInformationLayout->addWidget(mDesktopFrame);
    mInformationLayout->addWidget(mUsernameFrame);

    AboutLayout->addWidget(mTitleLabel);
    AboutLayout->addWidget(mInformationFrame);
    AboutLayout->addWidget(mActivationFrame);
    AboutLayout->addWidget(mTipLabel);
    AboutLayout->addSpacerItem(item);
    AboutLayout->addWidget(mPriTitleLabel);
    AboutLayout->addWidget(mPrivacyFrame);
    AboutLayout->addSpacing(0);
    AboutLayout->addWidget(mBtnFrame);
    AboutLayout->addWidget(mHoldTitleLabel);
    AboutLayout->addWidget(mHoldWidget);

    mverticalLayout->addWidget(Aboutwidget);
    mverticalLayout->addStretch();
}

QFrame *About::initPrivacyUi()
{
    QFrame *frame = new QFrame;
    setFrame_Box(frame);

    QHBoxLayout *mPrivacyLayout = new QHBoxLayout(frame);
    mPrivacyLayout->setContentsMargins(16, 16, 16, 16);
    mPrivacyLayout->setSpacing(48);
    mPriBtn = new KSwitchButton(frame);
    mPriBtn->setObjectName("privacysettings");

    FixLabel *mPriLabel_1 = new FixLabel(tr("Send optional diagnostic data"));
    LightLabel *mPriLabel_2 = new LightLabel(tr("By sending us diagnostic data, improve the system experience and solve your problems faster"));
    QVBoxLayout *mverticalLayout_2 = new QVBoxLayout;
    mverticalLayout_2->setSpacing(0);
    mverticalLayout_2->setContentsMargins(0, 0, 0, 0);
    mverticalLayout_2->addWidget(mPriLabel_1);
    mverticalLayout_2->addWidget(mPriLabel_2 );

    mPrivacyLayout->addLayout(mverticalLayout_2);
    mPrivacyLayout->addStretch();
    mPrivacyLayout->addWidget(mPriBtn);

    return frame;
}

QFrame *About::initActiveUi()
{
    QFrame *frame = new QFrame;
    setFrame_Box(frame);

    QHBoxLayout *mActivationLayout_1 = new QHBoxLayout(frame);
    mActivationLayout_1->setContentsMargins(16, 16, 16, 16);
    mActivationLayout_1->setSpacing(8);

    QGridLayout *mActivationLayout = new QGridLayout();
    mActivationLayout->setVerticalSpacing(8);

    mStatusLabel_1 = new FixLabel;
    mStatusLabel_1->setFixedSize(160, 30);
    mStatusLabel_2 = new LightLabel;
    mSequenceLabel_1 = new FixLabel;
    mSequenceLabel_1->setFixedSize(160, 30);
    mSequenceLabel_2 = new FixLabel;
    mSequenceLabel_2->setObjectName("sequence");
    mSequenceLabel_2->installEventFilter(this);
    mTimeLabel_1 = new FixLabel;
    mTimeLabel_1->setFixedSize(160, 30);
    mTimeLabel_2 = new FixLabel;
    mActivationBtn = new QPushButton;
    mActivationBtn->setObjectName("acivation");

    mActivationBtn->setFixedWidth(120);

    mActivationLayout->addWidget(mStatusLabel_1, 0, 0, 1, 1);
    mActivationLayout->addWidget(mStatusLabel_2, 0, 1, 1, 3,Qt::AlignLeft);
    mActivationLayout->addWidget(mSequenceLabel_1, 1, 0, 1, 1);
    mActivationLayout->addWidget(mSequenceLabel_2, 1, 1, 1, 3,Qt::AlignLeft);
    mActivationLayout->addWidget(mTimeLabel_1, 2, 0, 1, 1);
    mActivationLayout->addWidget(mTimeLabel_2, 2, 1, 1, 3,Qt::AlignLeft);

    mTimeLabel_1->hide();
    mTimeLabel_2->hide();
    mActivationLayout_1->addLayout(mActivationLayout);
    mActivationLayout_1->addWidget(mActivationBtn);

    return frame;
}

QFrame *About::initInfoBtnUi()
{
    QFrame *frame = new QFrame;
    setFrame_NoFrame(frame);

    QHBoxLayout *mBtnLyt = new QHBoxLayout(frame);
    mBtnLyt->setContentsMargins(16, 0, 0, 0);
    mTrialBtn = new KBorderlessButton(tr("<<Protocol>>"));
    mAndLabel = new FixLabel(tr("and"));
    mAgreeBtn = new KBorderlessButton(tr("<<Privacy>>"));
    mTrialBtn->setStyleSheet("text-decoration:underline");
    mAgreeBtn->setStyleSheet("text-decoration:underline");

    mBtnLyt->addWidget(mTrialBtn);
    mBtnLyt->addWidget(mAndLabel);
    mBtnLyt->addWidget(mAgreeBtn);
    mBtnLyt->addStretch();

    return frame;
}

QFrame *About::initHoldUi()
{
    QFrame *frame = new QFrame;
    frame->setMinimumSize(QSize(550, 120));
    frame->setMaximumSize(QSize(16777215, 120));
    frame->setFrameShape(QFrame::Box);

    QHBoxLayout *mHoldLayout = new QHBoxLayout(frame);
    mHoldLayout->setSpacing(8);
    mHoldLayout->setContentsMargins(16, 0, 8, 0);

    mQrCodeWidget = new QWidget;
    mQrCodeWidget->setFixedSize(104, 104);
    QHBoxLayout *qrhLyt = new QHBoxLayout(mQrCodeWidget);
    qrhLyt->setContentsMargins(0, 4, 0, 0);

    mHpLabel = new QLabel;
    mEducateLabel = new QLabel;
    mEducateIconLabel = new FixLabel;
    mEducateIconLabel->setFixedSize(96,96);

    mHpLabel->setWordWrap(true);
    mEducateLabel->setWordWrap(true);
    mHpLabel->setText(tr("Wechat code scanning obtains HP professional technical support"));
    mEducateLabel->setText(tr("See more about Kylin Tianqi edu platform"));

    mHpBtn = new QPushButton;
    mEducateBtn = new QPushButton;

#ifdef WIN32
    QPluginLoader loader("../HpQRCodePlugin/hp-qrcode-plugind.dll");
#else
    QPluginLoader loader("/usr/lib/x86_64-linux-gnu/hp-qrcode-plugin/libhp-qrcode-plugin.so");

#endif
    QObject *plugin = loader.instance();
    if (plugin) {
        app = qobject_cast<hp::QRCodeInterface*>(plugin);
        mQrCodeWidget->layout()->addWidget(app->createWidget(mQrCodeWidget));
    }
    else{
        qDebug() << "加载插件失败";
    }

    mHpBtn->setText(tr("Learn more HP user manual>>"));
    mHpBtn->setStyleSheet("background: transparent;color:#2FB3E8;font-family:Microsoft YaHei;"
                  "border-width:1px;text-decoration:underline;border-style:none none none;border-color:#2FB3E8;text-align: left");

    mEducateIconLabel->setPixmap(loadSvg(":/help-app.svg", mEducateIconLabel->size().width(), mEducateIconLabel->size().height()));
    mEducateBtn->setText(tr("See user manual>>"));
    mEducateBtn->setStyleSheet("background: transparent;color:#2FB3E8;font-size;font-family:Microsoft YaHei;"
                  "border-width:1px;text-decoration:underline;border-style:none none none;border-color:#2FB3E8;text-align: left");

    QVBoxLayout *hpLayout = new QVBoxLayout();
    hpLayout->setContentsMargins(0, 8, 0, 8);
    hpLayout->setSpacing(4);
    hpLayout->addStretch();
    hpLayout->addWidget(mHpLabel);
    hpLayout->addWidget(mHpBtn);
    hpLayout->addStretch();

    QVBoxLayout *eduLayout = new QVBoxLayout();
    eduLayout->setContentsMargins(0, 8, 0, 8);
    eduLayout->setSpacing(4);
    eduLayout->addStretch();
    eduLayout->addWidget(mEducateLabel);
    eduLayout->addWidget(mEducateBtn);
    eduLayout->addStretch();

    mHoldLayout->addWidget(mQrCodeWidget);
    mHoldLayout->addLayout(hpLayout);
    mHoldLayout->addSpacing(16);
    mHoldLayout->addStretch(1);
    mHoldLayout->addWidget(mEducateIconLabel);
    mHoldLayout->addLayout(eduLayout);
    mHoldLayout->addStretch(1);

    return frame;

}

QFrame *About::createFrame(FixLabel *label_1, LightLabel *label_2, LightLabel *label_3)
{
    QFrame *frame = new QFrame;
    frame->installEventFilter(this);
    setFrame_NoFrame(frame);

    QHBoxLayout *layout = new QHBoxLayout(frame);
    layout->setContentsMargins(0, 0, 16, 0);

    label_1->setFixedSize(120,30);

    label_2->setTextInteractionFlags(Qt::TextSelectableByMouse);
    label_2->setFixedHeight(30);

    layout->addWidget(label_1);
    layout->addSpacing(40);
    if (label_3) {
        layout->addWidget(label_2);
        layout->addWidget(label_3);
        layout->addStretch();
    } else {
        layout->addWidget(label_2, Qt::AlignLeft);
    }

    return frame;
}

/* 添加搜索索引 */
void About::initSearchText()
{
    //~ contents_path /About/version
    mVersionLabel_1->setText(tr("Version"));
    //~ contents_path /About/Kernel
    mKernelLabel_1->setText(tr("Kernel"));
    //~ contents_path /About/CPU
    mCpuLabel_1->setText(tr("CPU"));
    //~ contents_path /About/Memory
    mMemoryLabel_1->setText(tr("Memory"));
    //~ contents_path /About/Desktop
    mDesktopLabel_1->setText(tr("Desktop"));
    //~ contents_path /About/User
    mUsernameLabel_1->setText(tr("User"));
     //~ contents_path /About/Status
    mStatusLabel_1->setText(tr("Status"));
    mSequenceLabel_1->setText(tr("Serial"));
    mTimeLabel_1->setText(tr("DateRes"));
}

/* 初始化DBus对象 */
void About::initActiveDbus()
{
    activeInterface = QSharedPointer<QDBusInterface>(
        new QDBusInterface("org.freedesktop.activation",
                           "/org/freedesktop/activation",
                           "org.freedesktop.activation.interface",
                           QDBusConnection::systemBus()));
    if (activeInterface.get()->isValid()) {
        connect(activeInterface.get(), SIGNAL(activation_result(int)), this, SLOT(activeSlot(int)));
    }
}

void About::initCopyRightName()
{
#ifdef OPENKYLIN
    mCopyRightName = tr("Openkylin");
#else
    mCopyRightName = tr("KylinSoft");
#endif
}

void About::setConnect()
{
    connect(mHpBtn,&QPushButton::clicked,this,[=](){
        openIntelSlot(kHPFile);
    });

    connect(mEducateBtn, &QPushButton::clicked, this, [=]{
        openIntelSlot(kAboutFile);
    });

    connect(mActivationBtn, &QPushButton::clicked, this, &About::runActiveWindow);

    connect(mTrialBtn, &KBorderlessButton::clicked, this, [=]() {
        Common::buriedSettings(name(), "show trial exemption agreement", QString("clicked"));
        TrialDialog *mDialog = new TrialDialog(pluginWidget);
        mDialog->exec();
    });

    connect(mAgreeBtn, &KBorderlessButton::clicked, this, [=]() {
        Common::buriedSettings(name(), "show user privacy agreement", QString("clicked"));
        PrivacyDialog *mDialog = new PrivacyDialog(pluginWidget);
        mDialog->exec();
    });
}

/* 获取激活信息 */
void About::setupSerialComponent()
{
    if (!activeInterface.get()->isValid()) {
        qDebug() << "Create active Interface Failed When Get active info: " <<
            QDBusConnection::systemBus().lastError();
        return;
    }

    int status = 0;
    QDBusMessage activeReply = activeInterface.get()->call("status");
    if (activeReply.type() == QDBusMessage::ReplyMessage) {
        status = activeReply.arguments().at(0).toInt();
    }
    int trial_status = 0;
    QDBusMessage trialReply = activeInterface.get()->call("trial_status");
    if (trialReply.type() == QDBusMessage::ReplyMessage) {
        trial_status = trialReply.arguments().at(0).toInt();
    }

    QString serial;
    QDBusReply<QString> serialReply;
    serialReply = activeInterface.get()->call("serial_number");
    if (!serialReply.isValid()) {
        qDebug()<<"serialReply is invalid"<<endl;
    } else {
        serial = serialReply.value();
    }

    QDBusMessage dateReply = activeInterface.get()->call("date");
    if (dateReply.type() == QDBusMessage::ReplyMessage) {
        dateRes = dateReply.arguments().at(0).toString();
    }

    QDBusMessage trial_dateReply = activeInterface.get()->call("trial_date");
    QString trial_dateRes;
    if (trial_dateReply.type() == QDBusMessage::ReplyMessage) {
        trial_dateRes = trial_dateReply.arguments().at(0).toString();
    }
    mSequenceLabel_2->setText(serial);
    mSequenceLabel_2->setStyleSheet("color : #2FB3E8");
    if (dateRes.isEmpty()) {  //未激活
         if (!trial_dateRes.isEmpty()) {  //试用期
            mStatusLabel_2->setText(tr("Inactivated"));
            mStatusLabel_2->setStyleSheet("color : red ");
            mTimeLabel_1->setText(tr("Trial expiration time"));
            mTimeLabel_2->setText(trial_dateRes);
            mActivationBtn->setText(tr("Active"));
        } else {
             mTimeLabel_1->hide();
             mTimeLabel_2->hide();
             mStatusLabel_2->setText(tr("Inactivated"));
             mStatusLabel_2->setStyleSheet("color : red ");
             mActivationBtn->setText(tr("Active"));
         }
         activestatus = false;
    }  else {    //已激活
        mActivationBtn->hide();
        mTrialBtn->hide();
        mAndLabel->hide();
        mStatusLabel_2->setStyleSheet("");
        mStatusLabel_2->setText(tr("Activated"));
        mTimeLabel_2->setText(dateRes);
        mActivationBtn->setText(tr("Extend"));
    }
}

/* 获取内部版本号 */
void About::setVersionNumCompenent()
{
    version_t ver= kdk_system_get_version_detaile();
    mVersionNumberLabel_2->setText(ver.os_version);
    mPatchVersionLabel_2->setText(QString(ver.update_version));
}

/* 获取logo图片和版本名称 */
void About::setupVersionCompenent()
{
    QString versionPath = "/etc/os-release";
    QStringList osRes = readFile(versionPath);
    QString versionID;
    QString version;

    if (QGSettings::isSchemaInstalled(THEME_STYLE_SCHEMA)) {
            themeStyleQgsettings = new QGSettings(THEME_STYLE_SCHEMA, QByteArray(), this);
    } else {
        themeStyleQgsettings = nullptr;
        qDebug()<<THEME_STYLE_SCHEMA<<" not installed";
    }

    for (QString str : osRes) {
        if (str.contains("VERSION_ID=")) {
            QRegExp rx("VERSION_ID=\"(.*)\"$");
            int pos = rx.indexIn(str);
            if (pos > -1) {
                versionID = rx.cap(1);
            }
        }

        if (!QLocale::system().name().compare("zh_CN", Qt::CaseInsensitive)) {
            if (str.contains("VERSION=")) {
                QRegExp rx("VERSION=\"(.*)\"$");
                int pos = rx.indexIn(str);
                if (pos > -1) {
                    version = rx.cap(1);
                }
            }
        } else {
            if (str.contains("VERSION_US=")) {
                QRegExp rx("VERSION_US=\"(.*)\"$");
                int pos = rx.indexIn(str);
                if (pos > -1) {
                    version = rx.cap(1);
                }
            }
        }
    }

    if (!version.isEmpty()) {
        mVersionLabel_2->setText(version);
    }

    if (!versionID.compare(vTen, Qt::CaseInsensitive) ||
            !versionID.compare(vTenEnhance, Qt::CaseInsensitive) ||
            !versionID.compare(vFour, Qt::CaseInsensitive)) {
        mLogoLabel->setPixmap(loadSvg("://img/plugins/about/logo-light.svg", 130, 50)); //默认设置为light
        mThemePixmap = loadSvg("://img/plugins/about/logo-light.svg", 130, 50);
        if (themeStyleQgsettings != nullptr && themeStyleQgsettings->keys().contains(CONTAIN_STYLE_NAME_KEY)) {
            if (themeStyleQgsettings->get(STYLE_NAME_KEY).toString() == UKUI_DARK) { //深色模式改为dark
                mLogoLabel->setPixmap(loadSvg("://img/plugins/about/logo-dark.svg", 130, 50));
                mThemePixmap = loadSvg("://img/plugins/about/logo-dark.svg", 130, 50);
            }
            connect(themeStyleQgsettings,&QGSettings::changed,this,[=](QString changedKey) {  //监听主题变化
                if (changedKey == CONTAIN_STYLE_NAME_KEY) {
                    if (themeStyleQgsettings->get(STYLE_NAME_KEY).toString() == UKUI_DARK) {
                        mLogoLabel->setPixmap(loadSvg("://img/plugins/about/logo-dark.svg", 130, 50));
                        mThemePixmap = loadSvg("://img/plugins/about/logo-dark.svg", 130, 50);
                         emit changeTheme();
                    } else {
                        mLogoLabel->setPixmap(loadSvg("://img/plugins/about/logo-light.svg", 130, 50));
                        mThemePixmap = loadSvg("://img/plugins/about/logo-light.svg", 130, 50);
                        emit changeTheme();
                    }
                }
            });
       }
    } else {
        mActivationFrame->setVisible(false);
        mTrialBtn->setVisible(false);
        mAndLabel->setVisible(false);
        mLogoLabel->setPixmap(loadSvg("://img/plugins/about/logoukui.svg", 130, 50));
        mThemePixmap = loadSvg("://img/plugins/about/logoukui.svg", 130, 50);
    }

}

/* 获取桌面信息 */
void About::setupDesktopComponent()
{
    // 获取当前桌面环境
    QString dEnv;
    foreach (dEnv, QProcess::systemEnvironment()) {
        if (dEnv.startsWith("XDG_CURRENT_DESKTOP"))
            break;
    }

    // 设置当前桌面环境信息
    if (!dEnv.isEmpty()) {
        QString desktop = dEnv.section("=", -1, -1);
        if (desktop.contains("UKUI", Qt::CaseInsensitive)) {
            mDesktopLabel_2->setText("UKUI");
        } else {
            mDesktopLabel_2->setText(desktop);
        }
    }
    mDesktopLabel_2->setText("UKUI");
    ChangedSlot();

    QDBusConnection::systemBus().connect(QString(), QString("/org/freedesktop/Accounts/User1000"),
                                          QString("org.freedesktop.Accounts.User"), "Changed",this,
                                         SLOT(ChangedSlot()));
}

/* 获取CPU信息 */
void About::setupKernelCompenent()
{
    QString memorySize("0GB");
    QString cpuType;

    QString kernal = QSysInfo::kernelType() + " " + QSysInfo::kernelVersion();
    getTotalMemory();

    QDBusInterface *memoryDbus = new QDBusInterface("com.control.center.qt.systemdbus",
                                                             "/",
                                                             "com.control.center.interface",
                                                             QDBusConnection::systemBus(), this);
   if (memoryDbus->isValid()) {
       QDBusReply<QString>  result = memoryDbus->call("getMemory");
       qDebug()<<"memory :"<<result;
       if (result != "0") {
            memorySize.clear();
            memorySize.append(result + "GB" + mMemAvaliable);
       }
   }
   if (memorySize == "0GB")
       memorySize = mMemtotal + mMemAvaliable;

    qDebug()<<kernal;
    mKernelLabel_2->setText(kernal);
    mMemoryLabel_2->setText(memorySize);

    cpuType = Common::getCpuInfo();
    mCpuLabel_2->setText(cpuType);
}


/* 获取硬盘信息 */
void About::setupDiskCompenet()
{
    QProcess process_1;
    process_1.start("lsblk");
    process_1.waitForFinished();
    QString diskSize = process_1.readAllStandardOutput();
    foreach (QString diskResult, diskSize.split("\n")) {
        if (diskResult == NULL)
            continue;
        diskResult.replace(QRegExp("[\\s]+"), " ");
        diskInfo = diskResult.split(" ");

        if (diskInfo.count() >= 6 && diskInfo.at(5) == "disk" && (!diskInfo.at(0).contains("fd")) &&
               (diskInfo.at(2) !="1")) { //过滤掉可移动硬盘
            QStringList totalSize;
            totalSize.append(diskInfo.at(3));
            disk2.insert(diskInfo.at(0),totalSize); //硬盘信息分类存储，用以兼容多硬盘电脑
        }
    }
    QProcess process_2;
    process_2.start("df -Tl ");
    process_2.waitForFinished();
    QString diskSize2 = process_2.readAllStandardOutput();
    double availSize=0;
    QStringList diskInfo2;
    QMap<QString, QStringList>::iterator iter;
    for(iter=disk2.begin();iter!=disk2.end();iter++)
    {
        foreach (QString diskResult, diskSize2.split("\n")) {
            if (diskResult == NULL)
                continue;
            diskResult.replace(QRegExp("[\\s]+"), " ");
            diskInfo2 = diskResult.split(" ");
            if(diskInfo2.at(1).contains("overlay"))
                continue;
            if(diskInfo2.at(0).contains(iter.key())){
                availSize += diskInfo2.at(4).toInt();
            }
        }
        QString diskAvailable = QString::number((availSize/1024/1024), 'f', 1) + "G";
        iter.value().append(diskAvailable);
        availSize = 0;
    }

    int count = 0;
    for(iter=disk2.begin();iter!=disk2.end();iter++){
        if (disk2.size() == 1) {
            mDiskLabel_1->show();
            mDiskLabel_2->show();
            mDiskLabel_2->setText(iter.value().at(0) + "B (" + iter.value().at(1) + "B "+ tr("avaliable") +")");
        }
        else {
            mDiskLabel_1->hide();
            mDiskLabel_2->hide();
            QHBoxLayout * layout = new QHBoxLayout;
            QLabel *label = new FixLabel;
            label->setText(tr("Disk") + QString::number(count + 1));
            QLabel *diskLabel = new FixLabel;
            diskLabel->setText(iter.value().at(0) + "B (" + iter.value().at(1) + "B "+ tr("avaliable") +")");

            layout->addWidget(label);
            layout->addWidget(diskLabel);
            layout->addStretch();
            mDiskLayout->addLayout(layout);

        }
    }
}

/* 获取系统版本 */
void About::setupSystemVersion()
{
    if (!Common::isTablet()) {
        mDiskFrame->hide();
        mHoldWidget->hide();
        mHoldTitleLabel->hide();
        return;
    } else {
        mPrivacyFrame->hide();
        mHostNameFrame->hide();
        mPriTitleLabel->hide();
        mAndLabel->hide();
        mAgreeBtn->hide();
        mActivationFrame->hide();
        mTipLabel->hide();
        mTrialBtn->hide();
        mPatchVersionFrame->hide();
    }
}

void About::setHostNameCompenet()
{
    mHostNameLabel_2->setText(Common::getHostName());
}

void About::setPrivacyCompent()
{
    if (Common::isWayland()) {
        mPrivacyFrame->setVisible(false);
        mPriTitleLabel->setVisible(false);
        mAgreeBtn->setVisible(false);
        mAndLabel->setVisible(false);
        item->changeSize(0, 0);
        return;
    }
    QDBusInterface *PriDBus = new QDBusInterface("com.kylin.daq",
                                                             "/com/kylin/daq",
                                                             "com.kylin.daq.interface",
                                                             QDBusConnection::systemBus(), this);
    if (!PriDBus->isValid()) {
//        qDebug()<<"create pridbus error";
        return;
    }
    QDBusReply<int> reply = PriDBus->call("GetUploadState");
    mPriBtn->blockSignals(true);
    mPriBtn->setChecked(reply == 0 ? false : true);
    mPriBtn->blockSignals(false);

    connect(mPriBtn,&KSwitchButton::stateChanged ,this ,[=](bool status){
        Common::buriedSettings(name(), "Send optional diagnostic data", QString("settings"), status ? "true": "false");
        PriDBus->call("SetUploadState" , (status ? 1 : 0));
    } );
}

void About::showExtend(QString dateres)
{
    mTimeLabel_2->setText(dateres+QString("(%1)").arg(tr("expired")));
    mTimeLabel_2->setStyleSheet("color : red ");
    mActivationBtn->setText(tr("Extend"));
}

int About::ntpGethostbyname(char *servname)
{
    pthread_t t1 = 0;
    int ret = 0;

    struct dnsInfo dnsinfo;
    dnsinfo.servname = servname;
    dnsinfo.pid = pthread_self();

    ret = pthread_create(&t1, NULL, threadGetAddrInfo, (void *)&dnsinfo);
    if(ret == -1) {
        qDebug()<<"pthread_create error";
        return -1;
    }

    int btimeout = 1;
    ret = pthread_kill(t1, 0);
    if(0 == ret) {
        qDebug()<<"thread exist"; //子线程还在阻塞，主线程30ms后返回
        usleep(30*1000);//30ms
    } else if (ESRCH == ret) {
        qDebug()<<"thread not exist"; //子线程结束，主线程直接返回
        btimeout = 0;
    }

    return 0 == btimeout ? 0 : -1;
}

void *About::threadGetAddrInfo(void *arg)
{
    pthread_detach(pthread_self()); //设置为分离线程
    int ret = 0;
    struct dnsInfo *dnsinfo = (struct dnsInfo *)arg;
    struct addrinfo *rptr = NULL;
    struct addrinfo *iptr = NULL;
    char iphost[256] = { 0 };

    char *node = NULL;
    char *servname = NULL;

    if(dnsinfo->servname !=NULL)
        servname = strdup(dnsinfo->servname); //子线程中结构体数据使用strdup进行了保存，防止主线程退出后，传进来的指针数据丢失

    struct addrinfo hints;
    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_INET;
    hints.ai_socktype = SOCK_DGRAM;

    pthread_t pid_master = dnsinfo->pid;

    ret = getaddrinfo(servname, node, &hints, &rptr);

    if(ret != 0) {
        free(servname);
        free(node);
        pthread_exit(NULL);
    }

    ret = pthread_kill(pid_master, 0); //判断主线程是否退出
    if(0 == ret) {
        for (iptr = rptr; NULL != iptr; iptr = iptr->ai_next) {
            if (AF_INET != iptr->ai_family)
                continue;
            memset(iphost, 0, 256);
            if (NULL == inet_ntop(AF_INET, &(((struct sockaddr_in *)(iptr->ai_addr))->sin_addr), iphost, 256)) {
                continue;
            }
            mIpList.append(iphost);
        }
    }

    freeaddrinfo(rptr);
    free(servname);
    free(node);
    pthread_exit(NULL);
}

char *About::ntpdate(char *hostname)
{
    int portno = 123;     //NTP is port 123
    int maxlen = 1024;        //check our buffers
    int i;          // misc var i
    unsigned char msg[48]={(0 << 6) | (3 << 3) | (3 << 0), 0, 4, ((-6) & 0xFF), 0, 0, 0, 0, 0};    // the packet we send
    unsigned long  buf[maxlen]; // the buffer we get back
    struct protoent *proto;
    struct sockaddr_in server_addr;
    int s;  // socket
    long tmit;   // the time -- This is a time_t sort of

    proto = getprotobyname("udp");
    s = socket(AF_INET, SOCK_DGRAM, 0);
    if (-1 == s) {
        perror("socket");
        return NULL;
    }

    memset( &server_addr, 0, sizeof( server_addr ));
    server_addr.sin_family=AF_INET;
    server_addr.sin_addr.s_addr = inet_addr(hostname);

    server_addr.sin_port=htons(portno);

    i=sendto(s,msg,sizeof(msg),0,(struct sockaddr *)&server_addr,sizeof(server_addr));
    if (-1 == i) {
        perror("sendto");
        return NULL;
    }

    // 设置超时
    struct timeval timeout;
    timeout.tv_sec = 0;
    timeout.tv_usec = 300000;//微秒
    if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO_NEW, &timeout, sizeof(timeout)) == -1) {
        perror("setsockopt failed:");
        return NULL;
    }


    struct sockaddr saddr;
    socklen_t saddr_l = sizeof (saddr);
    i=recvfrom(s,buf,48,0,&saddr,&saddr_l);
    if (-1 == i) {
        perror("recvfr");
        return NULL;
    }

    tmit=ntohl((time_t)buf[4]);    // get transmit time

    tmit -= 2208988800U;

    return ctime(&tmit);
}

char *About::getntpdate()
{
    for (QString host : mNtphostName) {
        mIpList.clear();
        ntpGethostbyname(host.toLatin1().data());
        for (QString ip : mIpList) {
            qDebug()<<host<<"----->"<<ip<<"  :";
            char *IP = ip.toLatin1().data();
            char *result = ntpdate(IP);
            if (result) {
                return result;
            }
        }
    }
    return NULL;
}

int About::getMonth(QString month)
{
    if (month == "Jan") {
        return 1;
    } else if (month == "Feb") {
        return 2;
    } else if (month == "Mar") {
        return 3;
    } else if (month == "Apr") {
        return 4;
    } else if (month == "May") {
        return 5;
    } else if (month == "Jun") {
        return 6;
    } else if (month == "Jul") {
        return 7;
    } else if (month == "Aug") {
        return 8;
    } else if (month == "Sep" || month == "Sept") {
        return 9;
    } else if (month == "Oct") {
        return 10;
    } else if (month == "Nov") {
        return 11;
    } else if (month == "Dec") {
        return 12;
    }else {
        return 0;
    }
}

void About::reboot()
{
    QDBusInterface *rebootDbus = new QDBusInterface("org.gnome.SessionManager",
                                                    "/org/gnome/SessionManager",
                                                    "org.gnome.SessionManager",
                                                    QDBusConnection::sessionBus());

    rebootDbus->call("reboot");
    delete rebootDbus;
    rebootDbus = nullptr;
}

/* 处理文本宽度 */
void About::setLabelText(QLabel *label, QString text)
{
    QFontMetrics  fontMetrics(label->font());
    int fontSize = fontMetrics.width(text);
    if (fontSize > label->width()) {
        label->setText(fontMetrics.elidedText(text, Qt::ElideRight, label->width()));
        label->setToolTip(text);
    } else {
        label->setText(text);
        label->setToolTip("");
    }
}

/* 处理窗口缩放时的文本显示 */
bool About::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == mHostNameLabel_3) {
        if (event->type() == QEvent::MouseButtonPress){
            QMouseEvent * mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton ){            
                QString str = Common::getHostName();
                HostNameDialog *mdialog = new HostNameDialog(pluginWidget);
                QWidget *widget = qApp->activeWindow(); // 记录mainwindow的地址，exec之后，activeWindow会变成空值
                mdialog->exec();
                if (str !=  Common::getHostName()) {
                    QMessageBox *mReboot = new QMessageBox(widget);
                    mReboot->setIcon(QMessageBox::Warning);
                    mReboot->setText(tr("The system needs to be restarted to set the HostName, whether to reboot"));
                    mReboot->addButton(tr("Reboot Now"), QMessageBox::AcceptRole);
                    mReboot->addButton(tr("Reboot Later"), QMessageBox::RejectRole);
                    int ret = mReboot->exec();
                    switch (ret) {
                    case QMessageBox::AcceptRole:
                        sleep(1);
                        reboot();
                        break;
                    }
                    mHostNameLabel_2->setText(Common::getHostName());
                    Common::buriedSettings(name(), "change hostname", QString("settings"), Common::getHostName());
                }
            }
        }
    } else if ( obj == mSequenceLabel_2) {
        if (event->type() == QEvent::MouseButtonPress){
            QMouseEvent * mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton  && !mSequenceLabel_2->text().isEmpty()){
                Common::buriedSettings(name(), "show activation info", QString("clicked"));
                if (!dateRes.isEmpty())
                    compareTime(dateRes);
                StatusDialog *mDialog = new StatusDialog(pluginWidget);
                mDialog->mLogoLabel->setPixmap(mThemePixmap);
                connect(this,&About::changeTheme,[=](){
                    mDialog->mLogoLabel->setPixmap(mThemePixmap);
                });
                mDialog->mVersionLabel_1->setText(mVersionLabel_1->text());
                mDialog->mVersionLabel_2->setText(mVersionLabel_2->text());
                mDialog->mStatusLabel_1->setText(mStatusLabel_1->text());
                mDialog->mStatusLabel_2->setText(mStatusLabel_2->text());
                mDialog->mSerialLabel_1->setText(mSequenceLabel_1->text());
                mDialog->mSerialLabel_2->setText(mSequenceLabel_2->text());
                mDialog->mTimeLabel_1->setText(mTimeLabel_1->text());
                mDialog->mTimeLabel_2->setText(mTimeLabel_2->text());
                if (mDialog->mTimeLabel_2->text().contains(tr("expired"))) {
                    mDialog->mTimeLabel_2->setStyleSheet("color : red ");
                } else {
                    mDialog->mTimeLabel_2->setStyleSheet("");
                }

                if (!activestatus) {
                    mDialog->mTimeLabel_1->parentWidget()->hide();
                }
                mDialog->mExtentBtn->setText(mActivationBtn->text());
                connect(mDialog->mExtentBtn, &QPushButton::clicked, this, &About::runActiveWindow);
                mDialog->exec();
                return true;
            }
        }
    }
    return QObject::eventFilter(obj, event);
}

void About::fromOsRelseae()
{
    QString filePath = "/etc/os-release";
    QFile file(filePath);
    int pos = -1;
    if (file.exists()) {
        QStringList mCentent = readFile(filePath);
        for (QString str : mCentent) {
          if (str.contains("KYLIN_RELEASE_ID=")) {
              QRegExp rx("^KYLIN_RELEASE_ID=\"(.*)\"$");
              pos = rx.indexIn(str);
              if (pos > -1) {
                  mVersionNumberLabel_2->setText(rx.cap(1));
                  break;
              }
          }
        }
        if (pos == -1)
          mVersionNumberFrame->hide();
    } else {
        mVersionNumberFrame->hide();
    }
}

QStringList About::getUserDefaultLanguage()
{
    QString formats;
    QString language;
    QStringList result;

    unsigned int uid = getuid();
    QString objpath = "/org/freedesktop/Accounts/User"+QString::number(uid);

    QDBusInterface iproperty("org.freedesktop.Accounts",
                             objpath,
                             "org.freedesktop.DBus.Properties",
                             QDBusConnection::systemBus());
    QDBusReply<QMap<QString, QVariant> > reply = iproperty.call("GetAll", "org.freedesktop.Accounts.User");
    if (reply.isValid()) {
        QMap<QString, QVariant> propertyMap;
        propertyMap = reply.value();
        if (propertyMap.keys().contains("FormatsLocale")) {
            formats = propertyMap.find("FormatsLocale").value().toString();
        }
        if(language.isEmpty() && propertyMap.keys().contains("Language")) {
            language = propertyMap.find("Language").value().toString();
        }
    }
    qDebug()<<formats<<"---"<<language;
    result.append(formats);
    result.append(language);
    return result;
}

/* 读取/etc/os-release文件的内容 */
QStringList About::readFile(QString filepath)
{
    QStringList fileCont;
    QFile file(filepath);
    if (file.exists()) {
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
            qWarning() << "ReadFile() failed to open" << filepath;
            return QStringList();
        }
        QTextStream textStream(&file);
        while (!textStream.atEnd()) {
            QString line = textStream.readLine();
            line.remove('\n');
            fileCont<<line;
        }
        file.close();
        return fileCont;
    } else {
        qWarning() << filepath << " not found"<<endl;
        return QStringList();
    }
}

void About::getTotalMemory()
{
    const QString fileName = "/proc/meminfo";
    QFile meninfoFile(fileName);
    if (!meninfoFile.exists()) {
        printf("/proc/meminfo file not exist \n");
    }
    if (!meninfoFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        printf("open /proc/meminfo fail \n");
    }

    QTextStream in(&meninfoFile);
    QString line = in.readLine();
    float memtotal = 0;
    float memAvaliable = 0;
    int count = 0;

    while (!line.isNull()) {
        if (line.contains("MemTotal")) {
            line.replace(QRegExp("[\\s]+"), " ");

            QStringList lineList = line.split(" ");
            QString mem = lineList.at(1);
            memtotal = mem.toFloat();
            count++;
            if (count ==2) {
               break;
            } else {
                line = in.readLine();
            }
        } else if (line.contains("MemAvailable")) {
            line.replace(QRegExp("[\\s]+"), " ");

            QStringList lineList = line.split(" ");
            QString mem = lineList.at(1);
            memAvaliable = mem.toFloat();
            count++;
            if (count ==2) {
               break;
            }
        }
        else {
            line = in.readLine();
        }
    }

    memtotal = ceil(memtotal / 1024 / 1024);
    memAvaliable /= (1024 * 1024);
    // 向2的n次方取整
//    int nPow = ceil(log(memtotal)/log(2.0));
//    memtotal = pow(2.0, nPow);

    mMemtotal = QString("%1%2").arg(QString::number(memtotal)).arg("GB");
    mMemAvaliable = QString("%1%2%3%4%5").arg("(").arg(QString::number(memAvaliable, 'f', 1)).arg("GB").arg(tr("avaliable")).arg(")");
}

void About::compareTime(QString date)
{
    QString s1(getntpdate());
    QStringList list_1;
    QStringList list_2 = date.split("-");
    int year;
    int mouth;
    int day;
    if (s1.isNull()) {    //未连接上网络, 和系统时间作对比
        QString currenttime =  QDateTime::currentDateTime().toString("yyyy-MM-dd");
        qDebug()<<currenttime;
         list_1 = currenttime.split("-");
         year = QString(list_1.at(0)).toInt();
         mouth = QString(list_1.at(1)).toInt();
         day = QString(list_1.at(2)).toInt();
    } else {    //获取到网络时间
        s1.remove(QChar('\n'), Qt::CaseInsensitive);
        s1.replace(QRegExp("[\\s]+"), " ");   //把所有的多余的空格转为一个空格
        qDebug()<<"网络时间 : "<<s1;
        list_1 = s1.split(" ");
        year = QString(list_1.at(list_1.count() -1)).toInt();
        mouth = getMonth(list_1.at(1));
        day = QString(list_1.at(2)).toInt();
    }
    if (QString(list_2.at(0)).toInt() > year) { //未到服务到期时间
        mTimeLabel_2->setText(date);
    } else if (QString(list_2.at(0)).toInt() == year) {
        if (QString(list_2.at(1)).toInt() > mouth) {
            mTimeLabel_2->setText(date);
        } else if (QString(list_2.at(1)).toInt() == mouth) {
            if (QString(list_2.at(2)).toInt() > day) {
                mTimeLabel_2->setText(date);
            } else {   // 已过服务到期时间
                showExtend(date);
            }
        } else {
            showExtend(date);
        }
    } else {
        showExtend(date);
    }
}

void About::setFrame_Box(QFrame *frame)
{
    frame->setMinimumSize(QSize(550, 0));
    frame->setMaximumSize(QSize(16777215, 16777215));
    frame->setFrameShape(QFrame::Box);
}

void About::setFrame_NoFrame(QFrame *frame)
{
    frame->setMinimumSize(QSize(550, 30));
    frame->setMaximumSize(QSize(16777215, 30));
    frame->setFrameShape(QFrame::NoFrame);
}

void About::activeSlot(int activeSignal)
{
    if (!activeSignal) {
        setupSerialComponent();
    }
}

/* 打开激活窗口 */
void About::runActiveWindow()
{
    Common::buriedSettings(name(), "Activate the system or extend the service", QString("settings"));
    QString cmd = "kylin-activation";

    QProcess process(this);
    process.startDetached(cmd);
}

/* 获取用户昵称 */
void About::ChangedSlot()
{
    qlonglong uid = getuid();
    QDBusInterface user("org.freedesktop.Accounts",
                        "/org/freedesktop/Accounts",
                        "org.freedesktop.Accounts",
                        QDBusConnection::systemBus());
    QDBusMessage result = user.call("FindUserById", uid);
    QString userpath = result.arguments().value(0).value<QDBusObjectPath>().path();
    QDBusInterface *userInterface = new QDBusInterface ("org.freedesktop.Accounts",
                                          userpath,
                                        "org.freedesktop.Accounts.User",
                                        QDBusConnection::systemBus());
    QString userName = userInterface->property("RealName").value<QString>();
    mUsernameLabel_2->setText(userName);
}

void About::openIntelSlot(const QString &desktopFile)
{
    QDBusInterface ifc("com.kylin.AppManager",
                       "/com/kylin/AppManager",
                       "com.kylin.AppManager",
                       QDBusConnection::sessionBus());
    ifc.call("LaunchApp", desktopFile);
}

QPixmap About::loadSvg(const QString &path, int width, int height) {
    const auto ratio = qApp->devicePixelRatio();
    if (ratio >= 2) {
        width += width;
        height += height;
    } else {
        height *= ratio;
        width *= ratio;
    }
    QPixmap pixmap(width, height);
    QSvgRenderer renderer(path);
    pixmap.fill(Qt::transparent);

    QPainter painter;
    painter.begin(&pixmap);
    renderer.render(&painter);
    painter.end();

    pixmap.setDevicePixelRatio(ratio);
    return pixmap;
}
