/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#include "notice.h"
#include "appdetail.h"
#include "realizenotice.h"
#include "noticemenu.h"
#include "hoverwidget.h"
#include <QFileDialog>
#include <QTimer>
#include <QFileSystemWatcher>
#include <QSettings>
#include <QToolButton>
#include <QStandardPaths>
#include "interface/common.h"

#define NOTICE_SCHEMA         "org.ukui.control-center.notice"
#define NEW_FEATURE_KEY       "show-new-feature"
#define ENABLE_NOTICE_KEY     "enable-notice"
#define SHOWON_LOCKSCREEN_KEY "show-on-lockscreen"

#define AUTOSTARTDIR      "/etc/xdg/autostart/"

class ukToolButton : public QToolButton
{
public:
    ukToolButton(QWidget *parent = nullptr):
        QToolButton(parent)
    {

    }

protected:
   void paintEvent(QPaintEvent *event) override;
};

Notice::Notice() : mFirstLoad(true)
{
    pluginName = tr("Notice");
    pluginType = SYSTEM;
}

Notice::~Notice()
{
    if (!mFirstLoad) {
        qDeleteAll(vecGsettins);
        vecGsettins.clear();
    }
}

QString Notice::plugini18nName()
{
    return pluginName;
}

int Notice::pluginTypes()
{
    return pluginType;
}

QWidget *Notice::pluginUi()
{
    if (mFirstLoad) {
        pluginWidget = new QWidget;
        pluginWidget->setAttribute(Qt::WA_DeleteOnClose);
        mFirstLoad = false;

        if (QGSettings::isSchemaInstalled(NOTICE_SCHEMA)) {
            QByteArray id(NOTICE_SCHEMA);
            nSetting = new QGSettings(id, QByteArray(), this);
        }
        if (QGSettings::isSchemaInstalled(THEME_QT_SCHEMA)) {
            QByteArray id(THEME_QT_SCHEMA);
            mThemeSetting = new QGSettings(id, QByteArray(), this);
        }

        if (QGSettings::isSchemaInstalled(NOTIFICATION_DEMO_SCHEMA)) {
            QByteArray id(NOTIFICATION_DEMO_SCHEMA);
            mDisTrubSettings = new QGSettings(id, QByteArray(), this);
        }

        initUi(pluginWidget);
        //获取已经存在的动态路径
        listChar = listExistsCustomNoticePath();

        initSearchText();
        setupComponent();
        setupGSettings();
        initNoticeStatus();


        //设置白名单
        whitelist.append("kylin-screenshot.desktop");
        whitelist.append("peony.desktop");
        whitelist.append("ukui-flash-disk.desktop");
        whitelist.append("ukui-powermanagement-daemon.desktop");
        whitelist.append("kylin-system-update.desktop");
        whitelist.append("ukui-bluetooth.desktop");
        initOriNoticeStatus();

    }
    return pluginWidget;
}

const QString Notice::name() const
{
    return QStringLiteral("Notice");
}

bool Notice::isShowOnHomePage() const
{
    return false;
}

QIcon Notice::icon() const
{
    return QIcon::fromTheme("ukui-tool-symbolic");
}

bool Notice::isEnable() const
{
    return true;
}

void Notice::initUi(QWidget *widget)
{
    QVBoxLayout *mverticalLayout = new QVBoxLayout(widget);
    mverticalLayout->setSpacing(0);
    mverticalLayout->setContentsMargins(0, 0, 0, 0);

    mNoticeLabel = new TitleLabel;
    mNotFazeLabel = new TitleLabel;
    mNoticeLabel->setText(tr("Notice Settings"));
    mNotFazeLabel->setText(tr("NotFaze Mode"));

    UkccFrame *distrubFrame = new UkccFrame(widget);
    distrubFrame->setFrameShape(QFrame::NoFrame);
    LightLabel *notFazeLabel = new LightLabel(tr("(Notification banners, prompts will be hidden, and notification sounds will be muted)"));
    QHBoxLayout *distrubLyt = new QHBoxLayout(distrubFrame);
    distrubLyt->setContentsMargins(0, 0, 0, 0);
    distrubLyt->addWidget(mNotFazeLabel);
    distrubLyt->addSpacing(4);
    distrubLyt->addWidget(notFazeLabel, Qt::AlignLeft);

    SettingGroup *notFazeFrame = new SettingGroup(widget);
    initNotFaze(notFazeFrame);

    mGetNoticeWidget  = new SwitchWidget(tr("Get notifications from the app"));
    mApplistWidget = new SettingGroup;

    mverticalLayout->setSpacing(8);
    mverticalLayout->addWidget(distrubFrame);
    mverticalLayout->addWidget(notFazeFrame);
    mverticalLayout->addSpacing(24);
    mverticalLayout->addWidget(mNoticeLabel);
    mverticalLayout->addWidget(mGetNoticeWidget);
    mverticalLayout->addWidget(mApplistWidget);
    mverticalLayout->addStretch();

}

void Notice::initNotFaze(SettingGroup *widget)
{
    UkccFrame *autoOpenFrame = new UkccFrame(widget, UkccFrame::BorderRadiusStyle::Around, true);
    QHBoxLayout *autoLyt = new QHBoxLayout(autoOpenFrame);
    autoLyt->setContentsMargins(16, 0, 16, 0);
    QLabel *autoOpenLabel = new QLabel(tr("Automatically turn on"), autoOpenFrame);
    mOpenTimeHCombox = new QComboBox(autoOpenFrame);
    mOpenTimeHCombox->setFixedWidth(64);
    mOpenTimeMCombox = new QComboBox(autoOpenFrame);
    mOpenTimeMCombox->setFixedWidth(64);
    mCloseTimeHCombox = new QComboBox(autoOpenFrame);
    mCloseTimeHCombox->setFixedWidth(64);
    mCloseTimeMCombox = new QComboBox(autoOpenFrame);
    mCloseTimeMCombox->setFixedWidth(64);
    for (int i = 0; i < 24; i++) {
        mOpenTimeHCombox->addItem(QStringLiteral("%1").arg(i, 2, 10, QLatin1Char('0')));
        mCloseTimeHCombox->addItem(QStringLiteral("%1").arg(i, 2, 10, QLatin1Char('0')));
    }
    for (int i = 0; i < 60; i++) {
        mOpenTimeMCombox->addItem(QStringLiteral("%1").arg(i, 2, 10, QLatin1Char('0')));
        mCloseTimeMCombox->addItem(QStringLiteral("%1").arg(i, 2, 10, QLatin1Char('0')));
    }
    QLabel * label_1 = new QLabel(autoOpenFrame);
    label_1->setFixedWidth(20);
    label_1->setText(tr("to"));
    QLabel *label_2 = new QLabel(autoOpenFrame);
    label_2->setFixedWidth(4);
    label_2->setText(":");
    QLabel *label_3 = new QLabel(autoOpenFrame);
    label_3->setFixedWidth(4);
    label_3->setText(":");
    mAutoOpenSwitchBtn = new KSwitchButton(autoOpenFrame);
    autoLyt->addWidget(autoOpenLabel);
    autoLyt->addStretch();
    autoLyt->addWidget(mOpenTimeHCombox);
    autoLyt->addWidget(label_2);
    autoLyt->addWidget(mOpenTimeMCombox);
    autoLyt->addWidget(label_1);
    autoLyt->addWidget(mCloseTimeHCombox);
    autoLyt->addWidget(label_3);
    autoLyt->addWidget(mCloseTimeMCombox);
    autoLyt->addSpacing(24);
    autoLyt->addWidget(mAutoOpenSwitchBtn);

    mMultiScreenWidget = new SwitchWidget(tr("Automatically turn on when multiple screens are connected"));
    mFullScreenWidget = new SwitchWidget(tr("Automatically open in full screen mode"));
    mAllowAlarmrWidget = new SwitchWidget(tr("Allow automatic alarm reminders in Do Not Disturb mode"));

    widget->addWidget(autoOpenFrame);
    widget->addWidget(mMultiScreenWidget);
    widget->addWidget(mFullScreenWidget);
    widget->addWidget(mAllowAlarmrWidget);
}

void Notice::initSearchText()
{
    //~ contents_path /Notice/Notice Settings
    //~ contents_path /Notice/Get notifications from the app
    //~ contents_path /Notice/NotFaze Mode
}

void Notice::setupComponent()
{
    if (mDisTrubSettings) {
        if (mDisTrubSettings->keys().contains(AUTOSTART_KEY)) {
            bool ischecked = mDisTrubSettings->get(AUTOSTART_KEY).toBool();
            mAutoOpenSwitchBtn->setChecked(ischecked);
            if (mDisTrubSettings->keys().contains(TIMESTART_KEY) && mDisTrubSettings->keys().contains(TIMEEND_KEY)) {
                QString starttime = mDisTrubSettings->get(TIMESTART_KEY).toString();
                QString endtime = mDisTrubSettings->get(TIMEEND_KEY).toString();
                QStringList startlist = starttime.split(":");
                QStringList endlist = endtime.split(":");
                if (startlist.length() >= 2 && endlist.length() >= 2) {
                    mOpenTimeHCombox->setCurrentText(startlist.at(0));
                    mOpenTimeMCombox->setCurrentText(startlist.at(1));
                    mCloseTimeHCombox->setCurrentText(endlist.at(0));
                    mCloseTimeMCombox->setCurrentText(endlist.at(1));
                }
                setComBoxStatus(ischecked);
            }
        }

        if (mDisTrubSettings->keys().contains(PROJECTIONSCREEN_KEY))
            mMultiScreenWidget->setChecked(mDisTrubSettings->get(PROJECTIONSCREEN_KEY).toBool());

        if (mDisTrubSettings->keys().contains(FULLSCREEN_KEY))
            mFullScreenWidget->setChecked(mDisTrubSettings->get(FULLSCREEN_KEY).toBool());

        if (mDisTrubSettings->keys().contains(ALARMCLOCK_KEY))
            mAllowAlarmrWidget->setChecked(mDisTrubSettings->get(ALARMCLOCK_KEY).toBool());
    }
}

void Notice::setupGSettings()
{
    connect(mGetNoticeWidget, &SwitchWidget::stateChanged, [=](bool checked){
        Common::buriedSettings(name(), "whether to get the notification from the app", QString("settings"), checked ? "true" : "false");
        nSetting->set(ENABLE_NOTICE_KEY, checked);
        setHiddenNoticeApp(checked);
    });

    if (mDisTrubSettings) {
        if (mDisTrubSettings->keys().contains(AUTOSTART_KEY)) {
            connect(mAutoOpenSwitchBtn, &KSwitchButton::stateChanged, [=](bool checked) {
                Common::buriedSettings(name(), "whether auto turn on do not disturb mode", QString("settings"), checked ? "true" : "false");
                mDisTrubSettings->set(AUTOSTART_KEY, checked);
                setComBoxStatus(checked);
            });
        }

        if (mDisTrubSettings->keys().contains(PROJECTIONSCREEN_KEY)) {
            connect(mMultiScreenWidget, &SwitchWidget::stateChanged, [=](bool checked) {
                Common::buriedSettings(name(), "whether auto turn on do not disturb mode when multi screen connection", QString("settings"), checked ? "true" : "false");
                mDisTrubSettings->set(PROJECTIONSCREEN_KEY, checked);
            });
        }

        if (mDisTrubSettings->keys().contains(FULLSCREEN_KEY)) {
            connect(mFullScreenWidget, &SwitchWidget::stateChanged, [=](bool checked) {
                Common::buriedSettings(name(), "whether auto turn on do not disturb mode when full screen mode", QString("settings"), checked ? "true" : "false");
                mDisTrubSettings->set(FULLSCREEN_KEY, checked);
            });
        }

        if (mDisTrubSettings->keys().contains(ALARMCLOCK_KEY)) {
            connect(mAllowAlarmrWidget, &SwitchWidget::stateChanged, [=](bool checked) {
                Common::buriedSettings(name(), "whether allow alarm prompt at not disturb mode", QString("settings"), checked ? "true" : "false");
                mDisTrubSettings->set(ALARMCLOCK_KEY, checked);
            });
        }

        if (mDisTrubSettings->keys().contains(TIMESTART_KEY) && mDisTrubSettings->keys().contains(TIMEEND_KEY)) {
            connect(mOpenTimeHCombox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index){
                Q_UNUSED(index);
                Common::buriedSettings(name(), "open time(hour) when auto turn on do not disturb mode", QString("settings"), mOpenTimeHCombox->currentText());
                mDisTrubSettings->set(TIMESTART_KEY, QString("%1%2%3").arg(mOpenTimeHCombox->currentText()).arg(":").arg(mOpenTimeMCombox->currentText()));
                qDebug()<<mDisTrubSettings->get(TIMESTART_KEY).toString();
            });

            connect(mOpenTimeMCombox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index){
                Q_UNUSED(index);
                Common::buriedSettings(name(), "open time(minute) when auto turn on do not disturb mode", QString("settings"), mOpenTimeMCombox->currentText());
                mDisTrubSettings->set(TIMESTART_KEY, QString("%1%2%3").arg(mOpenTimeHCombox->currentText()).arg(":").arg(mOpenTimeMCombox->currentText()));
                qDebug()<<mDisTrubSettings->get(TIMESTART_KEY).toString();
            });

            connect(mCloseTimeHCombox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index){
                Q_UNUSED(index);
                Common::buriedSettings(name(), "close time(hour) when auto turn on do not disturb mode", QString("settings"), mCloseTimeMCombox->currentText());
                mDisTrubSettings->set(TIMEEND_KEY, QString("%1%2%3").arg(mCloseTimeHCombox->currentText()).arg(":").arg(mCloseTimeMCombox->currentText()));
                qDebug()<<mDisTrubSettings->get(TIMEEND_KEY).toString();
            });

            connect(mCloseTimeMCombox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index){
                Q_UNUSED(index);
                Common::buriedSettings(name(), "close time(minute) when auto turn on do not disturb mode", QString("settings"), mCloseTimeMCombox->currentText());
                mDisTrubSettings->set(TIMEEND_KEY, QString("%1%2%3").arg(mCloseTimeHCombox->currentText()).arg(":").arg(mCloseTimeMCombox->currentText()));
                qDebug()<<mDisTrubSettings->get(TIMEEND_KEY).toString();
            });
        }
    }
}

void Notice::initNoticeStatus()
{
    mGetNoticeWidget->blockSignals(true);
    mGetNoticeWidget->setChecked(nSetting->get(ENABLE_NOTICE_KEY).toBool());
    mGetNoticeWidget->blockSignals(false);

    mEnv = (QLocale::system().name() == "zh_CN" ? true : false);
    setHiddenNoticeApp(mGetNoticeWidget->isChecked());

}

void Notice::initOriNoticeStatus()
{
    QStringList appsDirs = QStringList(AUTOSTARTDIR) + QStandardPaths::standardLocations(QStandardPaths::ApplicationsLocation);

    for (QString path : appsDirs) {
        QDir dir(path);
        QStringList filters;
        filters<<QString("*.desktop");
        dir.setFilter(QDir::Files | QDir::NoSymLinks); // 设置类型过滤器，只为文件格式
        dir.setNameFilters(filters);  // 设置文件名称过滤器，只为filters格式
        initListUI(dir, path);
    }
    mApplist.clear();

}

void Notice::initListUI(QDir dir, QString path)
{
    for (uint i = 0; i < dir.count(); i++) {
        QString file_name = dir[i];  // 文件名称
        if (!whitelist.contains(file_name)) {
            continue;
        }
        QSettings* desktopFile = new QSettings(path + QLatin1Char('/') + file_name, QSettings::IniFormat);
        QString no_display, not_showin, only_showin, appname, appname_CN, appname_US, icon;
        if (desktopFile) {
           desktopFile->setIniCodec("utf-8");

           no_display = desktopFile->value(QString("Desktop Entry/NoDisplay")).toString();
           not_showin = desktopFile->value(QString("Desktop Entry/NotShowIn")).toString();
           only_showin = desktopFile->value(QString("Desktop Entry/OnlyShowIn")).toString();
           icon = desktopFile->value(QString("Desktop Entry/Icon")).toString();
           appname = desktopFile->value(QString("Desktop Entry/Name")).toString();
           appname_CN = desktopFile->value(QString("Desktop Entry/Name[zh_CN]")).toString();
           appname_US = desktopFile->value(QString("Desktop Entry/Name")).toString();
           delete desktopFile;
           desktopFile = nullptr;
        } else {
            return;
        }

        if (!not_showin.isEmpty()) {
            if (not_showin.contains("UKUI")) {
                continue;
            }
        }
        if (!only_showin.isEmpty()) {
            if (only_showin.contains("LXQt") || only_showin.contains("KDE")) {
                continue;
            }
        }
        if (mApplist.contains(appname)) {
            qDebug()<<appname;
            continue;
        }

        SwitchWidget *baseWidget = new SwitchWidget(mEnv ? appname_CN : appname_US, mApplistWidget);

        QLabel *iconLabel = new QLabel(baseWidget);
        iconLabel->setFixedSize(32, 32);
        if (file_name == "ukui-flash-disk.desktop")
            icon = "drive-removable-media-usb";
        setAppIcon(iconLabel, icon);
        connect(mThemeSetting, &QGSettings::changed, [=](const QString &key){
            if (key == "iconThemeName")
                setAppIcon(iconLabel, icon);

        });

        ukToolButton *setBtn = new ukToolButton(baseWidget);
        setBtn->setProperty("useButtonPalette", true);
        setBtn->setPopupMode(QToolButton::InstantPopup);
        setBtn->setFixedWidth(36);
        setBtn->setIcon(QIcon::fromTheme("view-more-horizontal-symbolic"));
        NoticeMenu *menu = new NoticeMenu(setBtn);
        setBtn->setMenu(menu);

        baseWidget->insertWidget(0, iconLabel);
        baseWidget->insertWidget(2, setBtn, 1, Qt::AlignRight);

        mApplistWidget->addWidget(baseWidget);

        //创建gsettings对象
        if(appname_CN.isEmpty()) {
            appname_CN = appname_US;
        }

        const QByteArray id(NOTICE_ORIGIN_SCHEMA);
        QGSettings *cnSettings = nullptr;
        QGSettings *usSettings = nullptr;

        QString cnpath = QString("%1%2%3").arg(NOTICE_ORIGIN_PATH).arg(appname_CN).arg("/");
        cnSettings = new QGSettings(id, cnpath.toUtf8().data(), this);

        QString uspath = QString("%1%2%3").arg(NOTICE_ORIGIN_PATH).arg(appname_US).arg("/");
        usSettings = new QGSettings(id, uspath.toUtf8().data(), this);

        QGSettings *settings = nullptr;
        settings = mEnv ? cnSettings : usSettings;

        baseWidget->setChecked(settings->get(MESSAGES_KEY).toBool());
        bool voiceEnable = settings->get(VOICE_KEY).toBool();
        bool detailEnable = settings->get(SHOW_DETAIL_KEY).toBool();
        bool showEnable = settings->get(SHOW_SCREENLOCK_KEY).toBool();
        QString style = settings->get(STYLE_KEY).toString();
        int mode = 0;
        if (style == "mutative") {
         mode = 0;
        } else if (style == "always") {
         mode = 1;
        } else if (style == "none") {
         mode = 2;
        }
        menu->initStaus(voiceEnable, detailEnable, showEnable, mode);

        connect(menu, &NoticeMenu::voiceSignals, [=](bool checked) {
            Common::buriedSettings(name(), "whether prompt sound during notification", QString("settings"), checked ? "true" : "false");
            cnSettings->set(VOICE_KEY, checked);
            usSettings->set(VOICE_KEY, checked);
        });

        connect(menu, &NoticeMenu::detailSignals, [=](bool checked) {
            Common::buriedSettings(name(), "whether to show the message content in the lock screen interface", QString("settings"), checked ? "true" : "false");
            cnSettings->set(SHOW_DETAIL_KEY, checked);
            usSettings->set(SHOW_DETAIL_KEY, checked);
        });

        connect(menu, &NoticeMenu::showSignals, [=](bool checked) {
            Common::buriedSettings(name(), "whether to show the notice in the lock screen interface", QString("settings"), checked ? "true" : "false");
            cnSettings->set(SHOW_SCREENLOCK_KEY, checked);
            usSettings->set(SHOW_SCREENLOCK_KEY, checked);
        });

        connect(menu, &NoticeMenu::styleBtnSignals, [=](int id) {
            QString str("mutative");
            if (id == 0)
                str = "mutative";
            else if (id == 1)
                str = "always";
            else if (id == 2)
                str = "none";
            Common::buriedSettings(name(), "set notice style", QString("settings"), str);
            cnSettings->set(STYLE_KEY, str);
            usSettings->set(STYLE_KEY, str);
        });


        connect(cnSettings, &QGSettings::changed, [=](QString key) {
            if (static_cast<QString>(MESSAGES_KEY) == key) {
                baseWidget->blockSignals(true);
                bool judge = cnSettings->get(MESSAGES_KEY).toBool();
                if (usSettings->get(MESSAGES_KEY).toBool() != judge)
                    usSettings->set(MESSAGES_KEY, judge);
                if (baseWidget->isChecked() != judge)
                    baseWidget->setChecked(judge);
                baseWidget->blockSignals(false);
            }
        });

        connect(usSettings, &QGSettings::changed, [=](QString key) {
            if (static_cast<QString>(MESSAGES_KEY) == key) {
                baseWidget->blockSignals(true);
                bool judge = usSettings->get(MESSAGES_KEY).toBool();
                if (cnSettings->get(MESSAGES_KEY).toBool() != judge)
                    cnSettings->set(MESSAGES_KEY, judge);
                if (baseWidget->isChecked() != judge)
                    baseWidget->setChecked(judge);
                baseWidget->blockSignals(false);
            }
        });

        connect(baseWidget, &SwitchWidget::stateChanged, [=](bool checked) {
            Common::buriedSettings(name(), appname_US + " whether to start notification", QString("settings"), checked ? "true" : "false");
            cnSettings->set(MESSAGES_KEY, checked);
            usSettings->set(MESSAGES_KEY, checked);
        });
    }}

void Notice::setAppIcon(QLabel *label, const QString &icon)
{
    QFileInfo iconfile(icon);
    QPixmap pixmap;
    if (!QString(icon).isEmpty()) {
        QIcon currenticon
            = QIcon::fromTheme(icon,
                               QIcon(QString("/usr/share/pixmaps/"+icon
                                             +".png")));
        pixmap = currenticon.pixmap(QSize(32, 32));
    } else if (iconfile.exists()) {
        pixmap = QPixmap(iconfile.filePath()).scaled(32, 32);
    } else {
        pixmap = QPixmap(QString(":/img/plugins/autoboot/desktop.png"));
    }
    label->setPixmap(pixmap);
}

void Notice::setHiddenNoticeApp(bool status)
{
    mApplistWidget->setVisible(status);
}

void Notice::setComBoxStatus(bool status)
{
    mOpenTimeHCombox->setEnabled(status);
    mCloseTimeHCombox->setEnabled(status);
    mOpenTimeMCombox->setEnabled(status);
    mCloseTimeMCombox->setEnabled(status);
}

void ukToolButton::paintEvent(QPaintEvent *event)
{
    QPalette paltte;
    paltte.setColor(QPalette::Button, paltte.base().color());
    this->setPalette(paltte);
    QToolButton::paintEvent(event);
}
