#include "universalinterface.h"
#include <unistd.h>

static UniversalInterface *s_globalInstance = nullptr;

UniversalInterface* UniversalInterface::self()
{
    if (!s_globalInstance) {
        s_globalInstance = new UniversalInterface();
    }
    return s_globalInstance;
}

UniversalInterface::UniversalInterface()
{

}

UniversalInterface::~UniversalInterface()
{
    if (s_globalInstance) {
        delete s_globalInstance;
        s_globalInstance = nullptr;
    }
    if (mUkccPanelGsettings) {
        delete mUkccPanelGsettings;
        mUkccPanelGsettings = nullptr;
    }
    if (mUkuiStyleGsettings) {
        delete mUkuiStyleGsettings;
        mUkuiStyleGsettings = nullptr;
    }
    if (mAccountUserDbus) {
        delete mAccountUserDbus;
        mAccountUserDbus = nullptr;
    }
    if (mFcitxInputMethodDbus) {
        delete mFcitxInputMethodDbus;
        mFcitxInputMethodDbus = nullptr;
    }
}

QGSettings* UniversalInterface::ukccPanelGsettings()
{
    if (!mUkccPanelGsettings) {
        const QByteArray id(UKCC_ID);
        if (QGSettings::isSchemaInstalled(id)) {
            mUkccPanelGsettings = new QGSettings(id, QByteArray());
        }
    }
    return mUkccPanelGsettings;
}

QGSettings* UniversalInterface::ukuiStyleGsettings()
{
    if (!mUkuiStyleGsettings) {
        const QByteArray id(UKUI_STYLE_ID);
        if (QGSettings::isSchemaInstalled(id)) {
            mUkuiStyleGsettings = new QGSettings(id, QByteArray());
        }
    }
    return mUkuiStyleGsettings;
}

QDBusInterface* UniversalInterface::accountUserDbus()
{
    if (!mAccountUserDbus) {
        unsigned int uid = getuid();
        QString objpath = QString("/org/freedesktop/Accounts/User") + QString::number(uid);
        mAccountUserDbus = new QDBusInterface("org.freedesktop.Accounts",
                                              objpath,
                                              "org.freedesktop.Accounts.User",
                                              QDBusConnection::systemBus());
    }
    return mAccountUserDbus;
}

QDBusInterface* UniversalInterface::fcitxInputMethodDbus()
{
    if (!mFcitxInputMethodDbus) {
        mFcitxInputMethodDbus =  new QDBusInterface("org.fcitx.Fcitx",
                                                    "/inputmethod",
                                                    "org.fcitx.Fcitx.InputMethod",
                                                    QDBusConnection::sessionBus());
    }
    return mFcitxInputMethodDbus;
}
