#ifndef UNIVERSALINTERFACE_H
#define UNIVERSALINTERFACE_H
#include <QByteArray>
#include <QGSettings/QGSettings>
#include <QDBusInterface>

class UniversalInterface
{
public:
    UniversalInterface();
    ~UniversalInterface();

    /**
     * @brief 获取通用接口类的实例
     *
     * @return UniversalInterface
     */
    static UniversalInterface *self();

    /**
     * @brief 获取org.ukui.control-center.panel.plugins的gsetting
     *
     * @return QGSettings
     */
    QGSettings *ukccPanelGsettings();

    /**
     * @brief 获取org.ukui.style的gsetting
     *
     * @return QGSettings
     */
    QGSettings *ukuiStyleGsettings();

    /**
     * @brief 获取当前用户的org.freedesktop.Accounts.User接口
     *
     * @return QDBusInterface
     */
    QDBusInterface *accountUserDbus();

    /**
     * @brief 获取org.fcitx.Fcitx.InputMethod接口
     *
     * @return QDBusInterface
     */
    QDBusInterface *fcitxInputMethodDbus();

private:
    const QByteArray UKCC_ID       = QByteArray("org.ukui.control-center.panel.plugins");
    const QByteArray UKUI_STYLE_ID = QByteArray("org.ukui.style");

private:
    QGSettings         *mUkccPanelGsettings    = nullptr;
    QGSettings         *mUkuiStyleGsettings    = nullptr;
    QDBusInterface     *mAccountUserDbus       = nullptr;
    QDBusInterface     *mFcitxInputMethodDbus  = nullptr;
};

#endif // UNIVERSALINTERFACE_H
